#! /bin/bash
###############  Copyright 2002 Trimble Navigation Limited #############
# $Id: eth0conf.sh,v 1.10 2004/10/12 21:07:14 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/etc/init.d/eth0conf.sh,v $
#
# eth0conf.sh
#
# Shell script to configure the Ethernet port eth0.
# This script is designed to be run by init, just after boot time, 
# attached to the Console port.  It informs the user of the current
# Ethernet setup and prompts for any desired changes to that
# configuration.  If no user interaction is detected for a short period,
# it times out and exits.
#
# Although designed to work from inittab, this script can also be run
# from getty or ssh logins, as needed.  Note that if running from
# ssh through the ethernet port, changing the ethernet configuration
# will probably cause difficulties by pulling the rug out from under 
# ssh.
########################################################################


# Ignore certain signals that could be generated by user input, 
# especially those that would abort or pause the script.  We always 
# want this to run until completion.
trap '' INT STOP TSTP

# Use extended glob pattern matching in this script, to allow certain 
# constructs like '*( )' (which means any number of spaces), etc.
shopt -s extglob


########################################################################
function NewConfig
########################################################################
# Update the network configuration file and activate the new settings.
# Input parameters:
#   $NewMode    (static or dhcp)
#   $NewEthIP
#   $NewEthNetmask
#   $NewEthGateway
#
# If this is a static setup, we determine the Network base address and
# the Broadcast address.  
# 
# Merge the new configuration data into the existing config file 
# /etc/sysconfig/interfaces.  We make a copy, with modifications, into 
# /tmp/interfaces and then install it back in /etc/sysconfig.  
# Finally we activate the new configuration.
#
{
  echo "Accepted - Updating configuration files"

  if [[ $NewMode = 'static' ]]
  then
    # Determine the network base and broadcast addresses from the
    # IP and Netmask.  This is done by performing arithmetic on the
    # individual octets (aka bytes).  We have to extract those into
    # separate shell variables first, using "set".

    # Separate the octets into $1..$8.
    IFS='.'
    set -- $NewEthIP $NewEthNetmask
    unset IFS
    #echo "$1 $2 $3 $4 $5 $6 $7 $8"

    # And'ing the matching parts of IP and Netmask gives 
    # the base network.
    # Note the unusual shell syntax here..  $(( )) evaluates some
    # arithmetic operands and returns the result as a string.
    network="$(($1 & $5)).$(($2 & $6)).$(($3 & $7)).$(($4 & $8))"
    #echo "Network=$network"

    # Or'ing IP with inverted netmask gives the Broadcast address.
    # Note the unusual shell syntax here..  $(( )) evaluates some
    # arithmetic operands and returns the result as a string.
    broadcast="$(($1 | (255 ^ $5))).$(($2 | (255 ^ $6))).$(($3 | (255 ^ $7))).$(($4 | (255 ^ $8)))"
    #echo "Broadcast=$broadcast"
  fi

  # The following is a loop for copying the existing interfaces file,
  # line by line, with filtering and substitutions.  We drop all
  # comments and blank lines.  Everything else gets transfered until
  # we see the line with 'iface eth0' on it.  That starts a section
  # containing multiple lines with specification options on them.
  # We generate new data for that portion.  Once we hit the iface eth0
  # line, we do alternate processing, skipping the original options
  # until something else comes along.
  ( 
    # '(' indicates a subshell to collect all output to the resulting
    # file.  Note the > redirection at end of subshell.

    # Don't let "read" separate on spaces.  Keeps lines intact with
    # original indentation.
    IFS=''    

    # Read all the original lines, one at a time.
    alt='' # Initialize state for the loop transfer operations.
    cat /etc/sysconfig/interfaces | 
    while read line
    do
      if [[ -z "$alt" ]]
      then
        # Basic processing.  Skip comments.  Preserve blank lines.
        # When we see 'iface eth0' output new data and switch to
        # alternate mode.  All other lines are passed through untouched.
        case $line in

        \#*) ;;

        '')  echo ;;

        *( )iface+( )eth0*) 
             if [[ $NewMode = 'static' ]]
             then
               echo "iface eth0 inet static"
               echo "  address $NewEthIP"
               echo "  network $network"
               echo "  netmask $NewEthNetmask"
               echo "  broadcast $broadcast"
               echo "  gateway $NewEthGateway"
             else
               echo "iface eth0 inet dhcp"
#               echo "    up   /etc/init.d/dhcpcd start"
#               echo "    down /etc/init.d/dhcpcd stop"
             fi
             # Default the MTU to 1500 for both static and dhcp.
             echo "  up ifconfig eth0 mtu 1500"
             echo
             alt=1
             ;;

        
        *) echo "$line" ;;

        esac
      else
        # Alternate processing mode.  We saw the 'iface eth0' line,
        # so now we are skipping the options introduced by that.
        # Still skip comments.  Drop blank lines.  When we reach
        # a line that would be the start of a new section, output that
        # line and switch back to normal processing.
        case $line in
        \#*) ;;
        '')  ;;
        *( )address*) ;;
        *( )network*) ;;
        *( )netmask*) ;;
        *( )broadcast*) ;;
        *( )gateway*) ;;
        *( )up*) ;;
        *( )down*) ;;
        *) echo "$line"
           alt='' 
           ;;
        esac
      fi
    done
  ) > /tmp/interfaces   # Std.output of subshell goes there.

  unset IFS
  #cat /tmp/interfaces

  # Apply the new settings, but only if we are on a fireball (ppc).
  # Don't try this on a workstation Linux because its configuration is
  # quite different.  
  echo "Applying new Network configuration"
  machine=$(uname -m)
  #echo $machine
  if [[ $machine = 'ppc' ]]
  then
    # Before switching network config, dismount any NFS mounts or they
    # will hang with stale handles.  Remount the NFS stuff after the
    # switch.
    if [[ -f /etc/init.d/umountnfs.sh ]]
    then
         /etc/init.d/umountnfs.sh
    fi

    # Stop existing network connections with the old configuration so
    # that the old config stuff is there to handle the shutdown.  Then
    # copy and activate the new configuration.
    /etc/init.d/networking stop
    mv /tmp/interfaces /etc/sysconfig/interfaces 
    /etc/init.d/networking start

    if [[ -f /etc/init.d/mountnfs.sh ]]
    then
         /etc/init.d/mountnfs.sh
    fi	 
  fi

} # end of NewConfig



########################################################################
function ReadYesNo
########################################################################
# Requests a yes/no answer from the user in response to a question.
# $1 is the question.
# $2 is the default response if the user merely hits return.
# $3 (if set) is the timeout delay.  If set and a timeout occurs,
#    it is the same as answering no.
#
# Returns 1 if no
# Returns 0 if yes
{
  timeout='' 
  if [[ $# == 3 ]]
  then
    timeout="-t $3"
  fi

  while :
  do
    echo -n "$1 (yes|no)[$2]: "
    if read $timeout response
    then
      if [[ -z "$response" ]]
      then
        response=$2 
      fi
      case "$response" in
      y*|Y*) return 0 ;;
      n*|N*) return 1 ;;
      b) /usr/local/fireball/bkdoor ;;
      *) echo -n $'\a' ;; # Bell Code
      esac
    else
      echo
      echo Timeout...
      return 1
    fi
  done

} # end of ReadYesNo



########################################################################
function GetIP
########################################################################
# Read an IP address in xx.xx.xx.xx format.
# $1 = name of the variable to assign the value to
# $2 = prompt string
# $3 = default value if user presses "return".
{
  varname=$1
  prompt=$2
  default=$3
  while :
  do
    echo -n "$prompt [$default]: "
    read response
    if [[ -z "$response" ]]
    then
      response=$3
    fi

    # Check response and return the value if it meets the
    # validity requirements.
    # Needed is a test for numeric range on each of the octets (0..255) 
    if echo $response | 
       grep -q -E '[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}'
    then
      eval $varname=$response
      return
    fi
  done
} # end of function GetIP


########################################################################
# Start of the main script.
# We run an infinite loop until the user says he's done or the first
# query times out.

while :
do

  # Get the current Ethernet configuration.  This is done by
  # Running a few system commands and parsing the outputs.  
  # This may still need more work to detect error conditions and provide
  # better default values.

  # Get the current assignment mode (static or dhcp) from the sysconfig
  # file "interfaces".  There will be a line like this:
  #   iface eth0 inet static
  # or
  #   iface eth0 inet dhcp
  # The fourth field is what we want.
  result=$(grep '^ *iface  *eth0' /etc/sysconfig/interfaces)
  set -- $result
  EthMode=$4	

  # Get Hardware MAC address.  ifconfig will product a line like:
  #   eth0 Link encap:Ethernet  HWaddr 00:60:35:00:B7:28
  # The fifth field is what we want.
  set -- $(/sbin/ifconfig eth0 | grep HWaddr)
  MacAddress=$5

  # Get the Current Ethernet IP and Netmask.  
  # If the network setup is okay, this will pop out of ifconfig in
  # the form
  #  inet addr:192.168.142.150  Bcast:192.168.142.255 Mask:255.255.255.0
  # We need fields 2 and 4, and have to drop the stuff before ':' in
  # each case.
  # If something is wrong with the current setup, then we won't get that
  # line from ifconfig.  If /etc/sysconfig/interfaces showed we are in
  # "static" mode, then we can read the IP and netmask from that file.
  # If all else fails, just default to 0.0.0.0 and 255.255.255.0.
  if /sbin/ifconfig eth0 | grep -q 'inet addr:'
  then
    set -- $(/sbin/ifconfig eth0 | grep 'inet addr:')
    EthIP=${2##*:}
    EthNetmask=${4##*:}
  elif [[ "$EthMode" = static ]]
  then
    set -- $(grep address /etc/sysconfig/interfaces)
    EthIP=$2
    set -- $(grep netmask /etc/sysconfig/interfaces)
    EthNetmask=$2
  else
    EthIP='0.0.0.0'
    EthNetmask='255.255.255.0'
  fi

  # In an functioning system, the ethernet gateway will be the
  # "default" route shown by /sbin/route.  We want the values to be
  # numeric, so we use the -n option.  That makes "default" into 0.0.0.0
  # so we search for that.  The line should look like:
  #   Destination  Gateway      Genmask   Flags Metric Ref    Use Iface
  #   0.0.0.0      155.63.20.1  0.0.0.0   UG    0      0        0 eth0
  # We grep for the starting 0.0.0.0 and then grab field number 2.
  # If that doesn't produce a default route then we try looking in
  # 'interfaces'.  There should be a specification if the mode is
  # static.  
  # If that fails then we just use the current IP as the default.
  gw=$(/sbin/route -n | grep '^0.0.0.0')
  if [[ -n "$gw" ]]
  then
    set -- $gw
    EthGateway=$2
  elif [[ $EthMode = "static" ]]
  then
    set -- $(grep gateway /etc/sysconfig/interfaces)
    EthGateway=$2
  else
    EthGateway=$EthIP
  fi


  echo ; echo ; echo ; echo
  echo $'\a############################################################'
  echo "Current EtherNet Port Configuration:" 
  echo   "  MAC address : $MacAddress"

  if [[ $EthMode = "static" ]]
  then
    echo "  Mode        : static IP"
  else
    echo "  Mode        : dhcp" 
  fi

  echo   "  IP          : $EthIP" 
  echo   "  Netmask     : $EthNetmask" 
  echo   "  Gateway     : $EthGateway" 
  #echo   "DNS Domain  : $DnsDomain" 
  #echo   "NameServer  : $DnsNameServer" 
  echo

  # Ask if the user want to change the setup.  Set the timeout to 10
  # seconds.  Quit if timeout or "no".
  if ! ReadYesNo  "Do you want to change Ethernet Configuration?" "no" 10
  then
    echo
    exit 
  fi

  # Get the new mode.
  while :
  do
    echo -n "New Mode (static|dhcp): "
    read NewMode
    case "$NewMode" in
    s*|S*) NewMode=static ; break ;;
    d*|D*) NewMode=dhcp ; break ;;
    esac
  done

  
  # Get the static parameters if needed.
  if [[ $NewMode = 'static' ]]
  then
    GetIP NewEthIP "New static IP Address" $EthIP 
    GetIP NewEthGateway "New Gateway Address" $EthGateway 
    GetIP NewEthNetmask "New Netmask" $EthNetmask 
  fi

  # Show off the new configuration state.
  echo 
  echo "New Configuration"
  if [[ $NewMode = 'static' ]]
  then
    echo "  Mode    : static IP"
    echo "  IP      : $NewEthIP"
    echo "  Gateway : $NewEthGateway"
    echo "  Netmask : $NewEthNetmask"
  else
    echo "  Mode    : dhcp"
  fi
  echo


  if ReadYesNo  "Accept New Configuration?" "yes"
  then
    NewConfig
  else
    echo "Changes Rejected"
  fi

done

########################################################################
# End of script.  We'll never get to here since the previous loop is
# infinite.  The only way to stop is by answering "no" at the first
# question above which triggers a direct exit.
########################################################################
