/********* Copyright (c) 2004 Trimble Navigation Limited ***************
 * $Id: clientAuthSample.c,v 1.2 2004/02/23 17:03:49 tom Exp $
 *
 * This source and the resulting compiled program can be freely
 * distributed.
 ***********************************************************************
 */

/***********************************************************************
 * clientAuthSample.c
 *
 * Program to open and receive data from a NetRS UDP stream port.
 * Invocation looks like:
 *
 *      clientAuthSample  IP_or_DNS_name  PortNum  Timeout
 *
 * Here's a specific example:
 *
 *      clientAuthSample  192.168.142.174  9997  5
 *
 * The first parameter is the IP address or DNS-name of the NetRS
 * device.
 *
 * Second parameter is the port number that is providing the
 * UDP stream, as set up in the IO configuration on the NetRS.
 *
 * The Timeout determines how often "keepalive" packets are sent by this
 * program to keep the data stream alive.  It should match the value
 * set on the I/O configuration page for the UDP Port.  Keepalive
 * packets will be sent approximately at intervals that are one third of
 * the timeout.
 *
 * This program can be compiled under Linux using the gcc command line:
 *
 *      gcc clientAuthSample.c -lcrypt -o clientAuthSample
 *
 ***********************************************************************
 */

#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <netdb.h>
#include <errno.h>
#include <stdio.h>

#include <unistd.h>



#define BUFSIZE 65536

struct message {
    int timeout ;
    int msgsize ;
    char buf[BUFSIZE] ;
} message ;


extern int errno ;


/* Forward declaration */
unsigned long long desEncrypt( unsigned long long challenge,
                               unsigned char *key
                             ) ;


/**********************************************************************/
int main( int argc, char **argv)
/**********************************************************************/
/* This is the main function.  It attempts to make contact with the
 * NetRS at the given address and UDP port number.  If a
 * client-authentication challenge is received it will query for a
 * key and make a response to trigger the stream.  Assuming all goes
 * well, it will read all packets that are sent over the channel, and
 * will display their lengths.
 */
{
  int s ;
  int port, timeout, interval ;
  struct sockaddr_in addr ;
  int len = sizeof(struct sockaddr_in) ;
  int err ;
  int fromlen, rlen ;
  struct hostent *hp ;
  char *server ;
  char streamData[255] ;
  char outputData[255] ;
  int authorized = 0 ;

  /* Must have all three parameters */
  if (argc != 4)
  {
    printf("usage: %s server port timeout\n",
    argv[0]) ;
    return 2 ;
  }

  server = argv[1] ;
  port = atoi( argv[2] ) ;
  interval = atoi( argv[3] ) ;

  /* Look up the server name to get the needed lower level
   * address.  This also accepts an IP address, which is just
   * copied into the resulting structure.
   * Returns a pointer to the structure, which perhaps is malloc'd
   * by the function.  We just keep and use the pointer.
   * Returns NULL pointer if there is a problem.
   */
  if ( (hp = gethostbyname( server )) == NULL )
  {
    fprintf( stderr, "Can't get host name\n" ) ;
    return 1 ;
  }
  memcpy( &addr.sin_addr, hp->h_addr, hp->h_length ) ;
  addr.sin_family = AF_INET ;
  addr.sin_port = htons( port ) ;

  printf( "Opening socket.\n" ) ;
  /* Create the UDP socket that we'll used for sending and receiving.
   * Note that at this point, the socket is not bound to any particular
   * port.
   */
  if ( (s = socket( AF_INET, SOCK_DGRAM, IPPROTO_UDP )) < 0 )
  {
    perror( "can't get socket" ) ;
    return 1 ;
  }


  timeout = time( NULL ) + (interval/3) ;

  /* Send a fake 100 byte message to the UDP port on NetRS.  This
   * should activate the port to start sending data back to the port
   * we will be listening to.
   *
   * When we attempt to send the message over the socket, the OS
   * automatically assigns the socket to a TCP port.  Once this is
   * done, subsequent writes and reads to that socket will use the
   * same port until the socket is closed.  The client on the other
   * end can detect the port (and IP number) from which we sent this
   * message, and will respond back to that Port & IP.
   */
  do
  {
    err = sendto( s,
                  (caddr_t)&message,
                  100,
                  0,
                  (struct sockaddr *)&addr,
                  len
                ) ;
    /* sendto() returns number of bytes actually sent, or -1 if an error
     * occured.  On an error due to a signal interrupting the process,
     * we'll try again.
     */
  } while ( (err == -1)
         && (errno == EINTR)
          ) ;


  if (err == -1)
  {
    perror( "sendto" ) ;
    return 1 ;
  }


  {
    /* Emit diagnostic describing the port we are bound to. */
    int len = sizeof(struct sockaddr) ;
    struct sockaddr_in saddr ;
    getsockname( s, (struct sockaddr *)&saddr, &len ) ;
    printf( "Socket bound to port %d.\n", ntohs(saddr.sin_port) ) ;
  }

  printf( "First send successful\nWaiting for reply...\n" ) ;


  /* Loop forever waiting for and receiving data from the bound socket.
   */
  do
  {
    fromlen = len ;
    rlen = recvfrom( s,
                     (caddr_t)streamData,
                     255,
                     0,
                     (struct sockaddr *)&addr,
                     &fromlen
                   ) ;
    printf( "%5d bytes ", rlen ) ;

    if (! authorized)
    {
      /* Not yet authorized. Check the first packet, which might
       * have an authorization challenge.
       */

      /* Regardless of whether that's an authorization challenge or
       * not, we'll only do this on the first packet received.  Thus
       * we set the authorized flag no matter what.
       */
      authorized = 1 ;

      /* If the first packet contains an authorization challenge ... */
      if ( strncmp( streamData, "NetRS SN", 8 ) == 0 )
      {
        unsigned long long challenge ;
        char rcvrType[20] ;
        char serialNumber[20] ;
        char sysName[20] ;
        char theKey[8] ;
        unsigned long long responseULL ;
        char responseHex[17] ;

        sscanf( streamData, "%s %s %s %llx",
                rcvrType,
                serialNumber,
                sysName,
                &challenge
              ) ;
        printf( "\nAuthorization challenge received from %s, %s, %s\n",
                rcvrType,
                serialNumber,
                sysName
              ) ;
        printf( "Challenge: %016llx\n", challenge ) ;

        printf( "Enter the seven-character key?  " ) ;
        scanf( "%7s", theKey ) ;

        responseULL = desEncrypt( challenge, theKey ) ;

        sprintf( responseHex, "%016llx", responseULL ) ;

        printf( " Response: %s\n", responseHex ) ;

        /* Send back the response. */
        sendto( s,
                (caddr_t)responseHex,
                strlen( responseHex ),
                0,
                (struct sockaddr *)&addr,
                len
              ) ;

        /* We've already processed this packet, so we need to go back
         * to the top of the loop to get the next one.
         */
        continue ;
      }

      /* Not an authorization packet, so presumably no authentication
       * was needed.  Just process the packet normally.
       */
    }

    printf( "from %d.%d.%d.%d, on local port %d \n",
             addr.sin_addr.s_addr >> 24,
            (addr.sin_addr.s_addr >> 16) & 0xff,
            (addr.sin_addr.s_addr >>  8) & 0xff,
             addr.sin_addr.s_addr        & 0xff,
            addr.sin_port
          ) ;
    memset( outputData, 0, 255 ) ;
    memcpy( outputData, streamData, rlen ) ;

    if ( time( NULL ) >= timeout )
    {
      err = sendto( s,
                    (caddr_t)&message,
                    1,
                    0,
                    (struct sockaddr *)&addr,
                    len
                  ) ;
      timeout = time( NULL ) + interval/3 ;
      printf( "Keepalive sent\n" ) ;
    }

  } while ( 1 ) ;

  return 0 ;

} /* end of main() */




/**********************************************************************/
unsigned long long keyToULL( unsigned char key[7] )
/**********************************************************************/
/* Accepts a 56-bit "key", which is an array of 7 bytes, using 8-bits
 * per byte.  This is coverted to a 64 bit key by breaking
 * up the 56 bits into a series of 7-bit values and concatenating
 * those into the resulting long-long value.  Note that every
 * 8th bit is zero in this result.
 * The result is returned as an unsigned long long (64 bit) value.
 */
{
  unsigned long long result ;

  result = 0 ;
  result += (                          key[0] >> 1 ) << 1 ;

  result <<= 8 ;
  result += (((key[0] & 0x01) << 6) + (key[1] >> 2)) << 1 ;

  result <<= 8 ;
  result += (((key[1] & 0x03) << 5) + (key[2] >> 3)) << 1 ;

  result <<= 8 ;
  result += (((key[2] & 0x07) << 4) + (key[3] >> 4)) << 1 ;

  result <<= 8 ;
  result += (((key[3] & 0x0f) << 3) + (key[4] >> 5)) << 1 ;

  result <<= 8 ;
  result += (((key[4] & 0x1f) << 2) + (key[5] >> 6)) << 1 ;

  result <<= 8 ;
  result += (((key[5] & 0x3f) << 1) + (key[6] >> 7)) << 1 ;

  result <<= 8 ;
  result += ( (key[6] & 0x7f)                      ) << 1 ;

  return result ;

} /* end of keyToULL() */



/**********************************************************************/
void expandFromULL( unsigned long long ull,
                    unsigned char bitArray[64]
                  )
/**********************************************************************/
/* This function converts a 64 bit value (encoded in an
 * unsigned long long) to an array of 64 bytes, with one bit per byte.
 * The mapping is:
 *      bitArray[ 0] = m.s.bit of ull
 *      bitArray[ 1] = next m.s.bit of ull
 *      ...
 *      bitArray[63] = l.s.bit of ull
 */
{
  int i ;

  for ( i = 63 ; i >= 0 ; --i )
  {
    bitArray[i] = ull & 1 ; /* Extract the l.s.bit */
    ull >>= 1 ; /* Drop that bit from ULL. */
  }

} /* end of expandFromULL() */



/**********************************************************************/
unsigned long long collapseToULL( unsigned char bitArray[64] )
/**********************************************************************/
/* This function is the inverse of expandFromULL().  It takes an input
 * array of 64 bytes, each of which of is either zero or 1.
 * This is compressed into a 64 bit unsigned long long and returned.
 */
{
  int i ;
  unsigned long long result ;

  for ( result = 0, i = 0 ; i < 64 ; ++i )
  {
    result <<= 1 ;          /* Make room for the new bit. */
    result += bitArray[i] ; /* Add the new bit. */
  }

  return result ;

} /* end of collapseToULL() */



/**********************************************************************/
unsigned long long desEncrypt( unsigned long long challenge,
                               unsigned char key[7]
                             )
/**********************************************************************/
/* Encrypts 64 bits of data from "challenge" using the
 * 7-byte key[].  Returns the resulting 64 bits of data as an
 * unsigned long long value.
 */
{
  unsigned char cryptKey[64] ;
  unsigned char desData[64] ;

  /* Convert the 56 bit key to 64 bits.
   * Expand the key into 64 bytes, 1 bit per byte.
   */
  expandFromULL( keyToULL( key ), cryptKey ) ;

  /* Make that the key used by the encrypt() function. */
  setkey( cryptKey ) ;

  /* Expand the 64 bit challenge into 64 bytes, 1 bit per byte. */
  expandFromULL( challenge, desData ) ;

  /* Encrypt desData in place.  The '0' indicate encrypt. */
  encrypt( desData, 0 ) ;

  /* Convert the 64 bits back into an unsigned long long
   * Return that value to the caller.
   */
  return collapseToULL( desData ) ;

} /* end of desEncrypt() */


/* end of file clientAuthSample.c */
