#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: dataFiles.cgi,v 1.14 2004/06/10 21:19:34 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/dataFiles.cgi,v $

########################################################################
# dataFiles.cgi
#     Script for displaying and managing logged data files.
#
# This script is called in one of several ways.  
#
#   no CGI parameters: 
#       Initial display at top level directory.
#       
#   ?dir=path
#       Show that directory path.
#
#   ?sort=name
#   ?sort=time
#   ?sort=timer
#   ?sort=size
#   ?sort=sizer
#       These request the directory list be sorted by the value.
#       time : earliest to latest
#       timer: latest to earliest
#       size : biggest to smallest
#       sizer: smallest to biggest
#
#   ?msg=MessageText
#       Used by the deleteFile.cgi script to indicate what file has been
#       recently deleted.
# 
# The parameters are commonly used in combination as with:
#   dataFiles.cgi?sort=size&dir=/abc/
#
# The Logged Data files are rooted at /var/gps/.  We want the user to
# be able to navigate the directory tree starting there, moving down
# into subdirectories and up to parent directory.  Files are displayed
# with icons, names, last-touched date/time, and size.  Also a delete
# button is available for each regular file.
# The order that files are listed in defaults to "by name", with
# sub-directories shown first followed by the regular files.  Clicking
# on the column titles sorts by that column: Name, Modified or Size.
# Subsequent clicks on Modified or Size reverses the order of the sort.
#
########################################################################

use strict ;
use CGI qw/:standard/;
use Fireball ;

titleWithHelp( 'Logged Data Files',
                         '/dataFilesInfo.html'
                       ) ;


my $Rootdir='/var/gps' ;

# Begin formatting the return document.
print header(-expires=>'+1s') ;


# If CGI parameter specifies a directory, use it.  Always end the
# directory with a '/'.  Note that this is a path relative to the
# logged files root, in $Rootdir.
my $dirpath = param('dir') ;
if (!$dirpath) 
{
  $dirpath = '/' ;
}
if ( $dirpath !~ '/$' )
{
  $dirpath .= '/' ;
}


# Determine how the entries should be sorted.  Default is by name.
my $sortBy = param('sort') ;
if ( (! $sortBy)
    || ( $sortBy ne 'size' 
      && $sortBy ne 'time' 
      && $sortBy ne 'timer' 
      && $sortBy ne 'sizer' 
       )
   )
{
  $sortBy = 'name' ;
}



# Make all links blue with no underlines, even if visited, etc.
# This is a document-level style specification, affecting all
# links in this frame.
print <<EOF ;
<style type="text/css">
 <!--
       a {color:blue ; text-decoration: none }
 -->
</style>
EOF


#for my $p (param())
#{
#  print $p, '=', param($p) , br() ;
#}

# If this page was called from the deleteFile.cgi page, a message
# will have been passed in via the parameter 'msg'.  Display that
# message.
if (my $message = param('msg'))
{
  print $message, br(), br() ;
}

# Display the diskspace available data.  Sadly, Perl doesn't have an
# easy way to get this via a system call.  We just run the "df" command
# and parse the output, which will look something like this:
#
#   Filesystem           1k-blocks      Used Available Use% Mounted on
#   rootfs                 6174820   3634344   2226808  62% /
#
# This is just split into a list, and the appropriate fields lifted
# from that.  'Filesystem' is field[0], 'on' is [6], etc.
my($usedSpace, $availSpace) = ( split " ", qx( df /var/gps ) )[9..10] ;

my ($journalSize) = qx( sudo /usr/local/fireball/ext3JournalSize ) ;
$usedSpace -= $journalSize ;
#printf( "<br />JournalSize: %.3f<br />", $journalSize ) ;


printf "<b>Logging Space:</b> %.3f Mbytes Used.  %.3f Mbytes Available",
       $usedSpace/1024, 
       $availSpace/1024 ;

print br(),br() ;

my $goodDirectory = chdir( $Rootdir.$dirpath ) ;

# If this is a valid directory, generate a list of sub-directories and 
# regular files.
my @dirfiles = () ;
my @regfiles = () ;

my %mtimes ;
my %sizes ;

if ($goodDirectory)
{
  # Go through the entire list of files in the current directory, and
  # extract those that are regular files.
  # Also create a list of the directory files.  
  # glob produces stuff in name order.  We'll re-sort later if needed.
  #
  # We skip the "lost+found" directory because it is not a place we
  # want the user to navigate to.  It's just an artifact of the
  # file system.


  for my $f (glob '*')
  {

    # Determine the status information on the file.  We only need
    # the size and modification timestamp.  
    # We use lstat() instead of stat() to properly deal with
    # symbolic links whose targets may have disappeared.  
    # The following few tests use the syntax "-d _",
    # where the "_" is perl shorthand for reusing the results from the
    # previous stat operation.

    my (undef, undef,undef, undef, undef, undef, undef, 
        $size, undef, $mtime, undef ) = lstat $f ; 

    if ( -f _ )
    {
      push( @regfiles, $f )
    }
    elsif ( -d _ )
    {
      # Don't show any files that are artifacts of our file system.
      # No CVS stuff (development systems only).
      # NO /tmp or /incoming or /lost+found
      push( @dirfiles, $f )
      unless ( ( $f eq 'incoming' and $dirpath eq '/' )
            or ( $f eq 'tmp' and $dirpath eq '/' ) 
            or ( $f eq 'lost+found' and $dirpath eq '/' ) 
            or ( $f eq 'CVS' )
             ) ;
    }
    elsif ( -l _ )
    {
      # This is a symlink.  It should point to a valid directory.
      # If it doesn't then we need to get rid of it!
      if ( -d $f )
      {
        push @dirfiles, $f ;
      }
      else
      {
        # Just remove it!  It's no good to anybody anyway.
        unlink $f ;
      }
    }

    $mtimes{$f} = $mtime ;

    $sizes{$f}  = $size ;
  }


  # We only sort the directories if "time" sort is selected.  If so,
  # we sort by oldest files first.  The sort function is fed the
  # Modification Age (-M) of the files.  That value is age in Days
  # (including fractional days) of each file.
  if ($sortBy eq 'time')
  {
    @dirfiles = sort {($mtimes{$b}) <=> ($mtimes{$a})} @dirfiles
  }

  if ($sortBy eq 'size')
  {
    @regfiles = sort {($sizes{$b}) <=> ($sizes{$a})} @regfiles
  }
  elsif ($sortBy eq 'sizer')
  {
    @regfiles = sort {($sizes{$a}) <=> ($sizes{$b})} @regfiles
  }
  elsif ($sortBy eq 'time')
  {
    @regfiles = sort {($mtimes{$b}) <=> ($mtimes{$a})} @regfiles
  }
  elsif ($sortBy eq 'timer')
  {
    @regfiles = sort {($mtimes{$a}) <=> ($mtimes{$b})} @regfiles
  }
}

my $subdircount  = scalar(@dirfiles) ;
my $regfilecount = scalar(@regfiles) ;

# Insert "..", the parent directory, in front of the list, but
# only if we aren't at the top of the tree.
if ($dirpath ne "/")
{
  splice @dirfiles, 0, 0, ( '..' ) ;
  (undef, undef,undef, undef, undef, undef, undef, 
        undef, undef, $mtimes{'..'}, undef ) = lstat ".." ; 
}

# Show the directory path we are displaying.
print b( "Directory path $dirpath" ) ;

if ( $goodDirectory )
{
print "<br />&nbsp;&nbsp;&nbsp;&nbsp;",
      $subdircount,
      " subdirector", 
      ($subdircount == 1) ? 'y' : 'ies',
      " and ",
      $regfilecount,
      " file",
      ($regfilecount == 1) ? '.' : 's.'
      ;
}
else
{
  print br(), b( "This directory does not exist, or cannot be read.") ;
}


# Start the table showing the files.
# Don't want any lines so border=0.
print p() ;
print '<Table border="0">' ;

# Headers for each column.
# Name, Modified and Size are made "bold" depending on which sorting
# option is chosen.
print Tr( td( "" ),
          td( a( {href=>"dataFiles.cgi?sort=name&dir=$dirpath" },
                 (($sortBy eq 'name') ? '<b>Name</b>' : 'Name')
               )
            ),
          td( {width=>"20"} ),
          td( {align=>'center'},
              a( {href=>( ($sortBy ne 'time')
                          ? "dataFiles.cgi?sort=time&dir=$dirpath"
                          : "dataFiles.cgi?sort=timer&dir=$dirpath"
                        ) 
                 },
                 (($sortBy eq 'time' || $sortBy eq 'timer') 
                  ? '<b>Modified</b>' 
                  : 'Modified')
               )
            ),
          td( {width=>"20"} ),
          td( {align=>'center'},
              a( {href=>( ($sortBy ne 'size')
                          ? "dataFiles.cgi?sort=size&dir=$dirpath" 
                          : "dataFiles.cgi?sort=sizer&dir=$dirpath" 
                        )
                 },
                 (($sortBy eq 'size' || $sortBy eq 'sizer') 
                  ? '<b>Size</b>' 
                  : 'Size')
               )
            ),
          td( {width=>"20"} ),
          td("Delete"),
        ) ;

# Put a horizontal line that spans all of the columns.
print Tr( td( {colspan=>8}, hr() ) ) ;

my @monthList = ("Jan","Feb","Mar","Apr","May","Jun",
                 "Jul","Aug","Sep","Oct","Nov","Dec") ;

# Go through each of the files/dirs and generate its line in the table.
# Each file gets an Icon.  A target is determined, which is used if the
# name or icon is clicked.
for my $f (@dirfiles, @regfiles)
{
  my $icon ;
  my $isRegularFile = 1 ;
  my $target ;
  my $delete ;

  # File is Actively logging, if it has a '.A' extension
  my $active = ( $f =~ m/\.A$/ ) ; 

  if ($f eq '..')
  {
    $icon = '/icons/back.gif' ;
    $isRegularFile = 0 ;
    # Target is the directory level just above this one.
    $target = $dirpath ;
    $target =~ s|[^/]*/$|| ; # Remove last element/ from path.
    $target="dataFiles.cgi?sort=$sortBy&dir=".$target ;
  }
  elsif (-d $f)
  {
    $icon = '/icons/folder.gif' ;
    $isRegularFile = 0 ;
    # Target is the subdirectory.
    $target="dataFiles.cgi?sort=$sortBy&dir=".$dirpath.$f."/" ;
  }
  else
  {
    # Pick an icon based on Text or Binary as indicated by -T.
    #$icon = (-T $f) ? '/icons/text.gif' : '/icons/binary.gif' ;
    $icon = '/icons/binary.gif' ;

    # Target is the URL to call to download this file.  It is actually
    # a pseudo-url of the form:
    #
    #    http://netrs/download/path/to/the/file
    #
    # Such a URL will be redirected by our web server due to 
    # scriptAliasMatch directives in the httpd configuration.
    # All such requests [starting with /download/] end up calling
    # perl-scripts/fileaccess/download.cgi, which determines the
    # path to the file from the original requesting URL.
    # Note that this link doesn't actually change the webpage displayed
    # in the browser.  We just want to download the file.
    $target = "/download" . $dirpath . CGI::escape($f) ;
  }
    
  
  # Convert modification time to date and time of day.
  my (undef,$min,$hour,$day,$month,$year) = gmtime($mtimes{$f}) ;
  $month= $monthList[$month] ;

  # Format nicely with a fixed-length field.  Put leading zeroes on all
  # variable length numbers.  Should look like: "11-JUL-2002 19:32"
  my $date = sprintf( "%02d-%s-%04d&nbsp;%02d:%02d", 
                     $day,
                     $month,
                     $year+1900,
                     $hour,
                     $min ) ;


  # If this is ".." then it is the parent directory.  Rename it to
  # make that clearer.
  $f = 'Parent Directory' if ($f eq '..') ;


  # Format the row of data associated with this file.
  # We use the "code" tag to force the strings on the web-page
  # to be in a mono-spaced font.
  # NOTE: This code is written in rather cryptic style for speed
  # reasons.  Given a large directory listing, this line is done
  # hundreds of times, and using the normal "cgi" constructs took a
  # tremendous amount of time.  I'm avoiding any function calls here.
  if ( !$active )
  {
    # File is not actively logging.  Emit the icon and filename with
    # the download link tags.
    print "<tr>\n<td> <a href=\"",
          $target,
          '"> <img border=0 src="',
          $icon,
          '" /> </a> </td> <td> <a href="', 
          $target, 
          '"> <code>',
          $f,
          '</code> </a></td>' ;
  }
  else
  {
    # An active file.  Suppress the download links.
    my $noA = $f ;
    $noA =~ s/\.A$// ;
    print "<tr>\n<td>",
          '<img border=0 src="',
          $icon,
          '" /> </td> <td>', 
          '<code>',
          $noA,
          '</code> </a></td>' ;
  }

  print '<td></td> <td> <code>',
        $date,
        '</code> </td>'
        ;

  # That will be all if this is a directory.  Otherwise we want to 
  # show filesize, and a delete button.
  # NOTE: See comment above about 'cryptic style'.  This is written
  # for execution speed, not clarity.
  if ( $isRegularFile )
  {
    my $size = $sizes{$f} ;
    $size = ($size >= 1024)
            ? sprintf( '%0.0fK', $size/1024 ) 
            : sprintf( '%d'    , $size      ) ;

    print '<td></td> <td align="right"> <code>',
          $size,
          '</code> </td> <td></td> <td align="center">' ;

    if ( !$active )
    {
      # Not an active file.  Okay to put up the Delete button.
      #print '<a href="/perl-scripts/fileaccess/deleteFile.cgi?sort=',
      #      $sortBy,
      #      "&delete=", 
      #      CGI::escape( $dirpath.$f ),
      print '<a href="javascript:rmfile(\'',
            $dirpath, 
            $f,
            '\')"><img border="0" alt="Delete" src="/images/redX.gif" />',
            '</a>' ;
    }
    else
    {
      # This file is actively logging.  Suppress the Delete button.
      print 'Logging' ;
    }
    print '</td>' ;
  }
 
  # End of the Row.
  print "</tr>\n" ;

}

# The last line of the table contains a delete button either to
# delete an empty directory, or to delete all of the files in this
# directory.  Suppress this at the top level directory.
if ( $subdircount  == 0
 and $regfilecount == 0
   )
{
  # There aren't any files here.  
  print '<tr>' ;
  print td( '&nbsp;'),
        td( {colspan=>6}, b("This directory is empty!") )
        ;
  if ( $dirpath ne "/" )
  {
    print td( {align=>'center'}, 
              a( { href => "javascript:RmDir( \'$dirpath\' )" },
                 img( { border=>'0',  
                        src=>'/images/redX.gif',
                        alt=>'Delete This Directory'
                      } 
                    )
               )
            ) ;
  }
  print '</tr>' ;
}
elsif ( $dirpath ne "/" )
{
  # If here, we have some files in the current directory.
  # We want to present the user with a button that would erase
  # this entire directory and all of the files it contains.
  # Before doing this, we want to put up an "alert" dialog asking
  # for confirmation.  This is handled via a javascript function
  # activated by the link in the image button.

  print '<tr>' ;
  print td( {colspan=>7, align=>'right'}, 
            b("Delete ALL FILES AND SUBDIRECTORIES in this directory.") 
          ) ;
  print '<td align="center">',
        '<a href="javascript:RmDir(\'',
        $dirpath,
        '\')"> <img border="0" alt="Delete" src="/images/redX.gif" />',
        '</a></td>' ;
  print '</tr>' ;

} 


# End of the Table.
print "</Table>\n" ;

########################################################################
# Put out the javascript function we'll need.
########################################################################

# Inside RmDir() we need to embed a timestamp.  This tags the page we
# are displaying so we can avoid deleting files that were modified
# between the time we display the page and the time we submit the
# RmDir request.  See usage in fileAccess/deleteFile.cgi
my $time=time() ;

print <<EOF ;
<script>

/********************************************************************/
function RmDir( dirname )
/********************************************************************/
{
  /* Check if the user really means to do the deletion.
   * confirm() returns true if user hits OKAY.
   */
  if ( confirm( 'Delete ALL FILES in directory ' 
                + dirname 
                + "\\nincluding ALL SUBDIRECTORIES?"
              ) 
     )
  {
    // Submit the cgi request to delete this directory.
    // Include the timestamp when this page was formatted, so we
    // can avoid deleting any files that changed AFTER we viewed
    // the directory.
    parent.loadDataFrame( 
                            '/perl-scripts/fileaccess/deleteFile.cgi'
                          + '?delete='
                          + escape(dirname)
                          + '&sortBy=$sortBy'
                          + '&time=$time'
                        ) ;
  }
} /* end of RmDir() */


/********************************************************************/
function rmfile( filename )
/********************************************************************/
{
    // Submit the cgi request to delete this file.
    parent.loadDataFrame( 
                          '/perl-scripts/fileaccess/deleteFile.cgi'
                          +'?delete=' 
                          + escape(filename)
                          +'&sortBy=$sortBy'
                        ) ;
} /* end of rmfile() */

</script>
EOF

print end_html() ;

EXIT:
# end of script dataFiles.cgi
