#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: setPort.cgi,v 1.44 2007/01/30 21:01:04 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/gpsadmin/setPort.cgi,v $

########################################################################
# setPort.cgi:
#
# Script to install new I/O port configurations.  The non-secured
# script /perl-scripts/portSetup.cgi is used to put up an HTML form
# for use in defining I/O ports.  When the user clicks OK, the
# form's parameters are passed into this script, which is gpsadmin
# secured.  Here we verify some items and then install the port by
# updating the gpssd configuration files, etc.
#
# If problems are found with the selected parameters, then an error
# message is posted and the portSetup.cgi script is re-invoked with
# the original parameters.
#
# The interface to the configuartion files is via a large hash
# structure, %allPortData{}.  The main keys are the names of the various
# ports, SerialPortX, TcpPortXXXX, RepPortXXXX or UdpPortXXXX.  Each of
# those indexed elements is a sub-hash with variously named elements,
# like 'service', 'protocol', 'baud', etc.  The process of creating or
# editing a port become a simple matter of reading in the full structure
# (with getPortSetup(), modifying the desired port characteristics, and
# then regenerating the configuration files with saveGpssdConfig().
# This abstraction layer hides the details of the configuration files,
# which is especially useful for PPP ports which have their information
# spread out over multiple config files.
########################################################################

use strict ;
use CGI qw/:standard/;


use lib '..' ;
use Fireball ;

########################################################################
# Begin formatting the return document.
########################################################################
startPage( 'Set Port' ) ;


# Read in the data from the configuration files and put in a Hash.
my %allPortData ;
getPortSetup( \%allPortData ) ;


######################################################################
# If we find any errors during subsequent processing, we set this
# string to indicate what's wrong.  That later triggers an
# alert box and shows the error on the page.
######################################################################
my $errorMessage = '' ;


# For debugging, display any parameters passed in.
# Disable in shipping code.
#if (param())
#{
#  print '<PRE>' ;
#  for my $i (param())
#  {
#    print "$i = ", param($i),"\n";
#  }
#  print '</PRE>' ;
#}

######################################################################
# Start processing the CGI parameters that define the port we are
# changing or creating.
######################################################################

# $thePort indicates which serial or network port being done.
# In input it may be "New TCP Port" or "New UDP Port" or one of the
# already defined ports of the form:
#   SerialPortX
#   TcpPortXXXX
#   UdpPortXXXX
my $thePort = param( 'thePort' ) ;


# Deal with the New ports if needed.
if ( defined(my $newPortNum = param('newPort')) )
{
  # This is a new TCP/UDP/REP port definition.
  # If everything checks out, we put the new value in place of
  # "$thePort and process as if this was a change.
  
  # The parameter newPort indicates the port number.
  # Must be a number in valid range.
  # Check if port number is numeric and in range.
  $newPortNum =~ s/^\s*// ;
  $newPortNum =~ s/\s*$// ;

  if ($newPortNum !~ /^\d+$/)
  {
    $errorMessage = "Port Number must be a positive number." ;
    goto ERROR ;
  }

  if ($newPortNum > 65535 or $newPortNum < 1024) 
  {
    $errorMessage = "Port Number must be between 1024 and 65535." ;
    goto ERROR ;
  }

  if ( $newPortNum == 4001 ) # Mfg Test socket
  {
    $errorMessage = "This Port Number is Reserved." ;
    goto ERROR ;
  }

  my $protocol ;
  if ($thePort =~ /TCP/i) 
  {
    $thePort = 'TcpPort'.$newPortNum ;
    $protocol = 'tcp' ;
  }
  elsif ($thePort =~ /UDP/i) 
  {
    $thePort = 'UdpPort'.$newPortNum ;
    $protocol = 'udp' ;
  }
  elsif ($thePort =~ /REP/i) 
  {
    $thePort = 'RepPort'.$newPortNum ;
    $protocol = 'rep' ;
  }

  # Check if port definition already exists.
  if (exists($allPortData{$thePort}))
  {
    $errorMessage = "$thePort is already defined." ;
    goto ERROR ;
  }
  # Create an entry in the port setup data for this new port.
  $allPortData{$thePort}{'protocol'} = $protocol ;

  # By default it will be enabled.
  $allPortData{$thePort}{'enable'} = 1 ;
}

# $thePort contains the name of the port (Serial/TCP/UDP/REP)
# to be modified (or created).  $thePort will be of the form:
# 'Serial Port 1' or 'TCP Port 1234' or 'UDP Port 1234' or 
# 'REP Port 1234'.
my ( $protocol, $portnum ) = split 'Port', $thePort ;
$protocol = lc($protocol) ;

my $authenticate = defined( param( 'Authenticate' ) ) ;

$allPortData{$thePort}{'authenticate'} = $authenticate ;

# Figure out what service is requested on this port.
my $theService = lc(param( 'service' )) ;
$theService = 'met' if ($theService eq 'met-tilt') ;
$theService = 'svstatus' if ($theService eq 'sv status') ;
$allPortData{$thePort}{'service'} = $theService ;

if ($protocol eq 'rep')
{
  # We can only have a single active REP protocol.  Since we
  # are activating this one, scan the others looking for duplicates.
  # Delete any other REP ports.
  for my $p ( keys %allPortData )
  {
    next if ($p eq $thePort) ;
    next if ($p eq 'MetControls') ;
    next if ($p eq 'AuthKey') ;
    if ($allPortData{$p}{'protocol'} eq 'rep')
    {
      delete( $allPortData{$p} ) ;
    }
  }
}


if ($theService eq 'ppp')
{
  ##################################################################
  # This is a PPP definition, and that means a serial port.  
  # We should now have CGI parameters defining:
  #   thePort
  #   baudrate 
  #   flowControl   (if not SerialPort1)
  #   localIP       
  #   remoteIP     
  #   connection = Direct|Modem
  #   initString0   (If Modem)
  #   initString1   (If Modem)
  #   initString2   (If Modem)
  #   resetString   (If Modem)
  ##################################################################

  # Verify the parameters we've just received, primarily the two
  # IP addresses.
  my $localIP = param( 'localIP' ) ;
  my $remoteIP = param( 'remoteIP' ) ;

  if (! $localIP or !$remoteIP)
  {
    $errorMessage = "Both IP addresses must be specified." ;
    goto ERROR ;
  }

  # Strip leading and trailing spaces
  $localIP =~ s/^\s*(\S*)\s*$/$1/ ;
  $remoteIP =~ s/^\s*(\S*)\s*$/$1/ ;

  if ( $localIP !~ m/^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/ 
    or $1 > 255
    or $2 > 255
    or $3 > 255
    or $4 > 255
    or $remoteIP !~ m/^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/ 
    or $1 > 255
    or $2 > 255
    or $3 > 255
    or $4 > 255
     )
  {
    $errorMessage = "Bad IP address." ;
    goto ERROR ;
  }

  $allPortData{$thePort}{'localip'} = $localIP ;
  $allPortData{$thePort}{'remoteip'} = $remoteIP ;

  $allPortData{$thePort}{'pppbaud'} = param('pppbaud') ;

  my $flow= param('pppflow') || 'None' ;
  $allPortData{$thePort}{'pppflow'} = $flow ;

  $allPortData{$thePort}{'connection'} = param('connection') ;

  # Modem Init Strings.  These are a bit tricky.
  # They are only applicable (and on the GUI page) if we have a
  # 'modem' style connection.  If the connection is 'direct' then
  # they won't be defined.  But we want the settings, if they were ever
  # entered, to be remembered in case we switch back to 'modem' mode.
  # 
  my $string = param('resetString') ;
  $allPortData{$thePort}{'reset'} = $string if (defined $string) ;

  $string = param('initString1') ;
  $allPortData{$thePort}{'init1'} = $string if (defined $string) ;

  $string = param('initString2') ;
  $allPortData{$thePort}{'init2'} = $string if (defined $string) ;

  $string = param('initString3') ;
  $allPortData{$thePort}{'init3'} = $string if (defined $string) ;

  # Since we are updating the ppp configuration, we have to force the
  # mgetty stuff to restart.  This is handled by gpssd, which is fed a
  # special parameter in field6 in gpssd.config.  This value toggles
  # between 1 and 10, and is solely used to determine if something has
  # changed which requires an mgetty restart.
  my $toggle = $allPortData{$thePort}{'ppptoggle'} || 1 ;
  $allPortData{$thePort}{'ppptoggle'} = ( ($toggle == 1) ? 10 : 1 ) ;

  # Update the ppp configuration files.  Must be done prior to updating
  # the gpssd.config, since that later operation is what restarts the
  # mgetty on the port.
  savePppConfig( \%allPortData ) ;

} # end of PPP processing

else

{
  # This is a stream definition (not PPP).  We need to assemble the 
  # data into the format required by gpssd.config.

  if ($protocol eq "serial") 
  {
    # Determine flow control settings.  N=None, H=Hardware,
    # S=Xon/Xoff(softwware).  # Serial Port 1 doesn't have flow
    # control and defaults to None.
    my $flow = 'none' ;
    if ($thePort ne 'SerialPort1')
    {
      $flow = param('flow') || 'none' ;
      $flow = 'xonxoff' if ($flow eq 'Xon/Xoff') ;
    }
    $allPortData{$thePort}{'flow'} = $flow ;

    $allPortData{$thePort}{'parity'} = param('parity') || 'none' ;
    $allPortData{$thePort}{'baud'} = param( 'baud' ) ;
  }
  elsif ($protocol eq 'udp')
  {
    # For UDP ports, the timeout popup must be read and translated
    # into the numeric value used in the config file.  This just
    # means dropping the " Second[s]" portion of the string,
    # that is, everything after the first space.
    my $timeout = param('UDPtimeout') ;
    $timeout =~ s/ .*$// ;
    $allPortData{$thePort}{'udptimeout'} = $timeout ;
  }

  ######################################################################
  # Process any parameters that are used to configure the
  # specific services
  ######################################################################
  if ( $theService eq 'rt17' )
  {
    $allPortData{$thePort}{'interval'}   = param('measInterval' ) ;

    $allPortData{$thePort}{'measurements'} 
                                         = param('Rt17Measurements') ;
    $allPortData{$thePort}{'positions'}  = param('Rt17Positions') ;
    $allPortData{$thePort}{'concise'}    = param('Rt17Concise') ;
    $allPortData{$thePort}{'iodeslips'}  = param('Rt17extras') ;
    $allPortData{$thePort}{'gpsdata'}    = param('Rt17svinfo') ;
    $allPortData{$thePort}{'rawgps'}     = param('Rt17Raw') ;
    $allPortData{$thePort}{'rawwaas'}    = param('Rt17Waas') ;
    $allPortData{$thePort}{'smoothcode'} = param('Rt17smoothCode') ;
    $allPortData{$thePort}{'smoothcarrier'} 
                                         = param('Rt17smoothCarrier') ;
    $allPortData{$thePort}{'useRt27'} 
                                         = param('Rt17useRt27') ;
  }
  ######################################################################
  elsif ($theService eq 'binex')
  {
    $allPortData{$thePort}{'interval'}  = param( 'measInterval' ) ;
    $allPortData{$thePort}{'smoothing'} = param('smoothing') || 0 ;
    $allPortData{$thePort}{'7f02'} = param('7f02') || 0 ;
    $allPortData{$thePort}{'7f03'} = param('7f03') || 0 ;
    $allPortData{$thePort}{'7f03Doppler'} = param('7f03Doppler') || 0 ;
    $allPortData{$thePort}{'7f03Slips'} = param('7f03Slips') || 0 ;
    $allPortData{$thePort}{'7f03RxClkOfsAlways'} =
                                      param('Always') || 0 ;
    $allPortData{$thePort}{'7f03RxClkOfsRollover'} =
                                    param('Rollover') || 0 ;

    $allPortData{$thePort}{'metaInterval'} = param('metaInterval') ;

    $allPortData{$thePort}{'siteDesc'} = param('siteDesc') ;
    $allPortData{$thePort}{'stationID'} = param('stationID') ;
    $allPortData{$thePort}{'antNum'} = param('antNum') ;
    $allPortData{$thePort}{'rcvrType'} = param('rcvrType') ;
    $allPortData{$thePort}{'rcvrNumber'} = param('rcvrNumber') ;
    $allPortData{$thePort}{'firmwareVersion'} 
                                            = param('firmwareVersion') ;
    $allPortData{$thePort}{'antXYZ'} = param('antXYZ') ;
    $allPortData{$thePort}{'antOffset'} = param('antOffset') ;

    $allPortData{$thePort}{'statusInterval'} = param('statusInterval') ;

    $allPortData{$thePort}{'metTilt'} = param('metTilt') ;
  }
  ######################################################################
  elsif ($theService eq 'svstatus')
  {
  }
  ######################################################################
  elsif ($theService eq 'ephemerides')
  {
  }
  ######################################################################
  elsif ($theService eq 'almanacs')
  {
  }
  ######################################################################
  elsif ($theService eq 'trimcom')
  {
    # We can only have a single trimcom TCPport active.  Since we
    # are activating this one, scan the others looking for duplicates.
    # Delete any other trimcom ports.
    for my $p ( keys %allPortData )
    {
      next if ($p eq $thePort) ;
      next if ($p eq 'MetControls') ;
      next if ($p eq 'AuthKey') ;
      if ($allPortData{$p}{'service'} =~ /trimcom/i)
      {
        delete( $allPortData{$p} ) ;
      }
    }
  } # Trimcom service
  ######################################################################
  elsif ($theService eq 'met')
  {
    # Updating Met-Tilt data 
    # Process the six commands and their schedules.
    # Put the data into the pseudo-port $allPortData{'MetControls'}. 
    # Each of those entries either contains "off' or a string
    # of the form   time:commandString
    # Time is in minutes, with zero encoding "once".

    for my $i (1..6) 
    {
      # Unless proven otherwise:
      $allPortData{'MetControls'}{$i} = 'off' ;

      my $sched = param( "Sched$i" ) ;
      my $cmd = param( "Cmd$i" ) ;
      $cmd =~ s/^  *// ;
      $cmd =~ s/ *$// ;
      # If there is a valid command, format and put it in the table.
      if (length($cmd) != 0 && $sched ne 'Off')
      {
        if ($sched eq 'Once') 
        {
            $sched = 0 ;
        }
        elsif ($sched =~ /minute/)
        {
          #  xx minutes
          $sched =~ s/ .*// ;
        }
        elsif ($sched =~ /second/)
        {
          #  xx seconds
          $sched =~ s/ .*// ;
          $sched .= "S" ;
        }
        else
        {
          #  xx hours
          $sched =~ s/ .*// ;
          $sched *= 60 ;
        }
        $allPortData{'MetControls'}{$i} = "$sched:$cmd" ;
      }
    }

    # Save the Met configuration data in the config file. 
    saveMetConfig( \%allPortData ) ;

    # Since we are setting up a Met port on this port we have to
    # be sure there isn't one already running on another port, and
    # cancel that.
    for my $p ( qw( SerialPort1 SerialPort2 SerialPort3 SerialPort4 ) )
    {
      next if ($p eq $thePort) ;
      if ($allPortData{$p}{'service'} =~ /met/i)
      {
        # Switch that port back to ppp default.
        $allPortData{$p}{'service'} = 'ppp' ;
      }
    }
 
  } # Met service
  ######################################################################
  elsif ( $theService eq 'ppp-dialout' )
  { 
    # Since we are setting up a ppp-dialout port on this port we have to
    # be sure there isn't one already running on another port, and
    # cancel that.
    for my $p ( qw( SerialPort1 SerialPort2 SerialPort3 SerialPort4 ) )
    {
      next if ($p eq $thePort) ;
      if ($allPortData{$p}{'service'} =~ /ppp-dialout/i)
      {
        # Switch that port back to ppp default.
        $allPortData{$p}{'service'} = 'ppp' ;
      }
    }
  }
  ######################################################################
  elsif ( $theService eq 'cmr' )
  { 
    # CMR configuration
    # The only option is the cmrStyle, which is either standard or
    # CMR-plus.  This is encoded as a 1 or a 2 in field 6 
    # field in the gpssd stream control line.
    $allPortData{$thePort}{'cmrStyle'} = param('cmrStyle') ;

  } # end of CMR
  ######################################################################
  elsif ( $theService eq 'rtcm' )
  { 
    # RTCM configuration

    # Version is either v2.1, v2.2, v2.3, v3.0
    # Convert and save without the 'v'.
    my $version = param('rtcmVersion' ) ;
    $version =~ s/^v// ;
    $allPortData{$thePort}{'version'} = $version ;

    # RTCM style is either DGPS, RTK, DGPSplusRTK or Custom:....
    my $style = lc( param('rtcmStyle') || 'dgps' ) ;
    if ($style eq 'custom')
    {
      $style .= ':' ;
      $style .= '1,' if param('Type1') ;
      $style .= '2,' if param('Type2') ;
      $style .= '3,' if param('Type3') ;
      $style .= '9,' if param('Type9') ;
      $style .= '14,' if param('Type14') ;
      $style .= '18+19,' if param('Type1819') ;
      $style .= '22,' if param('Type22') ;
      $style .= '23+24,' if param('Type2324') ;
      $style .= '59.3,' if param('Type59.3') ;
      $style .= '59.13,' if param('Type59.13') ;
      $style =~ s/,$// ; # Drop the final comma
    }
    $allPortData{$thePort}{'rtcmStyle'} = $style ;

  } # end of RTCM

} # end of non-PPP ports.



# Diagnostic output.  Emit a formatted version of the new port data.
#use lib '../sysadmin' ;
#use Commands ;
#Commands::formatPort( $thePort, \%allPortData ) ;


# Regenerate gpssd.config and inform the streaming daemon
# to examine the configuration for changes.
saveGpssdConfig( \%allPortData ) ;


# After a timeout, go back to main Sessions screen.
print <<EOF;
    <h2>New Port Configuration Accepted.</h2>
    <Script>
    parent.loadAfterDelay( "portSummary.cgi", 1000) ;
    </Script>
EOF


ERROR:
# If an error message was found earlier during CGI processing,
# post an alert message to the user.
if ($errorMessage)
{
  print <<EOF ;
<Script> 
alert( "$errorMessage" ) ;
</Script>
EOF

  # Also display the message on the top of the screen, in case
  # the javascript alert system doesn't work.
  print h3("$errorMessage") ;

  # Switch back to the original portSetup.cgi page.  In order to do this
  # we need to reconstruct the original URL, containing all of the
  # parameters that came into this script.  PortSetup.cgi will redraw 
  # the page, using the supplied parameters.  The list of parameters
  # must be of the form:
  #   portSetup.cgi ? parm1 = value & parm2 = value & parm3 = ...
  # In that, I put in spaces for clarity, but they aren't there in the
  # real thing.  Note that the parameter separators start with ? and
  # then switch to '&'.  The parameter values must be CGI::escaped to
  # convert spaces, etc., into URL acceptable sequences like %20.

  my $returnURL = '/perl-scripts/portSetup.cgi' ;

  my $startC = '?' ;
  for my $p (param())
  {
    $returnURL .= $startC ;
    $returnURL .= $p.'='.CGI::escape( param($p) ) ;
    $startC = '&' ;
  }

  #print h2( $returnURL ) ;

  print <<EOF ;
  <Script>
    parent.loadAfterDelay( "$returnURL", 1000) ;
  </Script>
EOF

}


EXIT:
print end_html() ;

########################################################################
# End of file setPort.cgi
########################################################################
