#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: portSetup.cgi,v 1.112 2007/01/30 22:14:41 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/portSetup.cgi,v $

########################################################################
# portSetup.cgi:
#
# Script to allow creation and editing of serial port and internet 
# port configurations using an HTML form.  This script manages the form.
# It is non-secured, so anyone can access it and view to setup
# parameters on existing ports.  When the user clicks OK or Delete,
# the form data is passed to the scripts gpsadmin/setPort.cgi or
# gpsadmin/deletePort.cgi.  Those scripts ARE secured with gpsadmin
# permissions, and implement the actual creation or deletion of the
# ports in the configuration files.
#
# CGI calling conventions.
#   portSetup.cgi
#       No parameters: Fresh start with serial port 1
#
#   portSetup.cgi?restart=Port
#       Fresh start with the specified port.
#       The Port parameter can have one of several forms:
#           Serial_Port_1
#           TCP_Port_1234
#           UDP_Port_1234
#           New_TCP_Port 
#           New_UDP_Port 
#           New_REP_Port 
#
#   portSetup.cgi?(full list of parameters from a form)
#       An update request to redraw the form information.
#       This is done when a parameter is changed that requires
#       new option widgets, etc.  
#
# The information used to populate the form, especially for existing
# ports, comes from the hash structure %allPortData. This is created
# by a call to getPortSetup().  The structure is indexed by port names,
# like SerialPortX, TcpPortXXXX, UdpPortXXXX or RepPortXXXX.  Each of
# those indexed elements is a sub-hash with variously named elements,
# like 'service', 'protocol', 'baud', etc.  
########################################################################

use strict ;
use CGI qw/:standard/;

use Fireball ;

titleWithHelp( 'Port Configuration',
               '/ioconfigGeneral.html'
             ) ;


# For debugging, display any parameters passed in.
#if (param())
#{
#  print '<PRE>' ;
#  for my $i (param())
#  {
#    print "$i = ", param($i),"\n";
#  }
#  print '</PRE>' ;
#}




########################################################################
sub showRefpos
########################################################################
# Subroutine used by two page formats below to show the reference
# station configuration.  This consists of identification numbers and
# strings, as well as Lat/Lon/Alt and XYZ.
# The only difference between the two places this is show is in the 
# first identification number, which can be different between CMR
# stations and RTCM stations.  To handle that difference, a single
# parameter is expected by this function
#   $_[0] eq 'cmr'  or  $_[0] eq 'rtcm'
# We check for 'cmr' and assume the other state if not cmr.
{
  my %refStationData ;
  getRefStationData( \%refStationData ) ;

  my $ecefX = $refStationData{ 'x' } ;
  my $ecefY = $refStationData{ 'y' } ;
  my $ecefZ = $refStationData{ 'z' } ;

  my $rtcmID= $refStationData{ 'rtcmid' } ;
  my $cmrID = $refStationData{ 'cmrid' } ;
  my $rtcmDesc = $refStationData{ 'rtcmdesc' } ;

  print table( {border=>0},
              Tr( td( {align=>'right'}, b( 'Station Identifier:' )),
                  td( ($_[0] eq 'cmr') ? $cmrID : $rtcmID ),
                ),
              Tr( td( {align=>'right'}, b('Description:') ),
                  td( {colspan=>3},
                      $rtcmDesc ),
                ),
             ) ;


  print table( {border=>0},
              Tr( td( {align=>'center', colspan=>4}, 
                  b('Reference Position') ),
                ),
              Tr( td( {align=>'right'}, b('Lat:') ),
                  td( {align=>'right'},
                      '<code>',
                      sprintf( '&nbsp;%02d&deg;%02d\'%08.5f" %s',
                               $refStationData{'latdeg'},
                               $refStationData{'latmin'},
                               $refStationData{'latsec'},
                               $refStationData{'latns'},
                             ),
                      '</code>'
                    ),
                  td( {align=>'left'}, 
                      b('&nbsp;&nbsp;&nbsp;ECEF-X:') ),
                  td( {align=>'right'},
                      '<code>', 
                      sprintf( '%12.3f', $ecefX), 
                      '</code> meters'
                    ),
                ),
              Tr( td( {align=>'right'}, b('Lon:') ),
                  td( {align=>'right'},
                      '<code>',
                      sprintf( '%03d&deg;%02d\'%08.5f" %s',
                               $refStationData{'londeg'},
                               $refStationData{'lonmin'},
                               $refStationData{'lonsec'},
                               $refStationData{'lonew'},
                             ),
                      '</code>'
                    ),
                  td( {align=>'left'}, b('&nbsp;&nbsp;&nbsp;ECEF-Y:') ),
                  td( {align=>'right'},
                      '<code>',
                      sprintf( '%12.3f', $ecefY), 
                      '</code> meters'
                    ),
                ),
              Tr( td( {align=>'right'}, b('Height:') ),
                  td( {align=>'right'},
                      '<code>',
                      sprintf( '%.3f', $refStationData{'height'} ),
                      '</code> meters'
                    ),
                  td( {align=>'left'}, b('&nbsp;&nbsp;&nbsp;ECEF-Z:') ),
                  td( {align=>'right'},
                      '<code>',
                      sprintf( '%12.3f', $ecefZ), 
                      '</code> meters'
                    ),
                ),
             ) ;

  # Grab the current position from the sysinfo facility.
  open( SYSINFO, '/usr/local/fireball/sysinfo -p pos_info |' );
  # First line of the form:
  # 246021.8, 1225 : 37.3891238781 -122.0368426604 -4.921
  # time      week   lat           lon             height
  my (undef, undef, undef, 
      $currentLat, $currentLon, $currentHeight) 
                                              = split( ' ', <SYSINFO>) ;
  close( SYSINFO ) ;

  #print "$currentLat, $currentLon, $currentHeight <br />" ;

  my ($currentX, $currentY, $currentZ ) 
     = split( ' ',
              qx( /usr/local/fireball/ecefConvert lla $currentLat $currentLon $currentHeight ) 
            ) ;

  #print "$currentX, $currentY, $currentZ <br />" ;
  my $deltaX = $currentX - $ecefX ;
  my $deltaY = $currentY - $ecefY ;
  my $deltaZ = $currentZ - $ecefZ ;

  my $diff = sqrt( $deltaX * $deltaX
                 + $deltaY * $deltaY
                 + $deltaZ * $deltaZ
                 ) ;
  # The $diff value determined above is the approximate error between
  # the entered reference position and the current GPS-based position.
  # If that's above a threshold, then we need to warn the user.  The
  # primary reason for this is that some reference station formatters
  # (RTCM in particular) will not produce any corrections if the error
  # is above some limit, probably 200m for RTCM.  Here we use a somewhat
  # arbitrary threshold of 100meters.  This is mainly intended to catch
  # the situation where a user enters something catstrophically wrong,
  # like a slipped degrees digit, or the wrong N/S/E/W identifier.
  if ($diff > 100)
  {
    printf( '<br /><h3>Reference position is different from '
            .'measured position by %.0d meters!</h3>', 
            $diff  
          ) ;
  }
} ## end of showRefpos()



# Read in the data from the configuration files and put in a Hash.
my %allPortData ;
getPortSetup( \%allPortData ) ;


########################################################################
# Create the list of items for the popup Port menu.
# Sort to put the Serial Ports first, followed by REP,TCP & UDP.
# Include elements for creating new TCP/UDP/REP port definitions.
########################################################################

# create a sorted list of all of the port data keys.
my @portList = sort keys %allPortData ;
# drop the non-port entries in the hash.
@portList = grep !/MetControls|AuthKey/, @portList ;
# Sort with Serial ports first.
@portList = ( (grep /Serial/, @portList),
              (grep !/Serial/, @portList) 
            ) ;

my %portLabels ;
for my $p (@portList)
{
  my $label = $p ;
  $label =~ s/Port/ Port / ;
  $portLabels{$p} = $label ;
}

# Add the menu items for defining new ports.
push @portList, 'New REP Port' ;
$portLabels{'New REP Port'} = 'New REP Port' ; 
push @portList, 'New TCP Port' ;
$portLabels{'New TCP Port'} = 'New TCP Port' ; 
push @portList, 'New UDP Port' ;
$portLabels{'New UDP Port'} = 'New UDP Port' ; 



########################################################################
# If no parameters have been supplied, use a default port and
# initialize as if restarted on that.
########################################################################
if (!param())
{
  param( 'restart', 'SerialPort1' ) ;
}

########################################################################
# $thePort indicates which serial or network port is being shown on 
# the current page.  This value will be set in one of several places
# below.
########################################################################
my $thePort ;


########################################################################
# If this is a restart, determine the current settings on the
# requested port (serial or TCP or UDP or REP) and set all CGI
# parameters for that port.
########################################################################
if (param('restart')) 
{
  $thePort = param('restart') ;
  CGI::delete_all() ;

  if ($thePort =~ /New REP/)
  {
    param( 'service', 'Binex' ) ;
  }
  elsif ($thePort =~ /New /)
  {
    param( 'service', 'RT17' ) ;
  }
  else
  {
    # Determine all of the parameters associated with the port named
    # in "$thePort"
    # This is done in one of several cases, depending on the type
    # of port selected.

    my $service = $allPortData{$thePort}{'service'} ;
    $service = 'RTCM' if ($service eq 'Rtcm') ;
    $service = 'Met-Tilt' if ($service eq 'Met') ;
    $service = 'SV Status' if ($service eq 'SvStatus') ;
    $service = 'PPP' if ($service eq 'ppp') ;
    $service = 'PPP-Dialout' if ($service eq 'ppp-dialout') ;

    if ($thePort =~ /SerialPort/)
    {
      param( 'baud', $allPortData{$thePort}{'baud'} ) ;
      param( 'pppbaud', $allPortData{$thePort}{'pppbaud'} ) ;

      param( 'parity', $allPortData{$thePort}{'parity'} ) ;
    }
    # Set the default service for use in the radio buttons on the 
    # page.
    param( 'service', $service ) ;


    param( 'Authenticate', $allPortData{$thePort}{'authenticate'} ) ;

    # Set the UDP timeout parameter.  This is only used on UDP ports,
    # and will be ignored on other types.  The numeric value is
    # converted to a string to match what is used on the popup selector
    # that shows up later.  Search for "UDPtimeout".
    my $timeout = $allPortData{$thePort}{'udptimeout'} || 10 ;
    if ($timeout == "1") 
    {
      $timeout .= " Second" ;
    }
    else
    {
      $timeout .= " Seconds" ;
    }
    param( "UDPtimeout", $timeout ) ;


  }
}
else
{ 
  # This isn't a restart request, so it must be an Update request.
  # That means that some control (probably the "service") has changed
  # which requires modifying the displayed options.  We have to redraw
  # the page, using the new settings, putting defaults in any control
  # elements that aren't already in use.  For example, if a port was
  # sending CMR data, and is switched to RT17, then the RT17-specific
  # options would have to be shown instead of the CMR-specific options.
  # All that we have to do in this case is set $thePort and continue to
  # the normal redraw of the page below.
  # Update requests can also come in due to errors found during OK
  # processing in gpsadmin/setPort.cgi.

  $thePort=param('thePort') ;
}



# If at this point, we are going to display a form for
# controlling a specific port (in $thePort) with a specific
# service.
my $theService = param( 'service' ) ;


########################################################################
# The Javascript functions needed by this page.
########################################################################
print <<EOF ;
<Script>


/**********************************************************************/
function Update()
/**********************************************************************/
/* One of the mode-selection radio buttons has been clicked (or
 * something like that.) This requires us to redraw the page.
 */
{

  parent.submitForm( document.theForm, '/perl-scripts/portSetup.cgi' ) ;

} /* end of Update() */


/**********************************************************************/
function restart()
/**********************************************************************/
/* The Port to modify is selected by a popupmenu, which is populated
 * with all of the existing sessions.  When this selector is changed, we
 * re-format the entire screen with that particular port's
 * information.  This is triggered by an onChange condition on the
 * popup, which calls the javascript restart() function defined here.
 * That function calls the cgi script with the special parameter
 * 'restart=portname'.
 */
{
  selectedIndex=document.theForm.thePort.selectedIndex ;
  selectedName=document.theForm.thePort.options[selectedIndex].value ;
  parent.loadDataFrame( 'portSetup.cgi?restart=' 
                        + escape(selectedName)
                      ) ;

} /* end of restart() */


/**********************************************************************/
function Okay()
/**********************************************************************/
/* Invoked when the OK button is pressed.
 * This just submits the entire form with all parameters to the
 * setPort.cgi script, after some error checking.
 */
{
  f = document.theForm ;


  /* Determine the selected service type.
   * In the normal case, there is an array of buttons and we
   * have to scan the array looking for the checked value.
   * The value of that button is used as the service.
   * But in at least one case, there is a single button which
   * Javascript won't regard as an array.  So we just assume it
   * is checked and use its label.
   */
  if (f.service.length != null)
  {
    for (index = 0 ; index < f.service.length ; ++index)
    {
      if ( f.service[index].checked )
      {
        service = f.service[index].value ;
        break ; // done when we have found Binex service.
      }
    }
  }
  else
    service = f.service.value ;


  // If service type is Binex and smoothing is checked and
  // measInterval values is 0.1 or 0.2 or 0.5 then alert.
  // Note that this code assumes (for ease of coding) a specific
  // order to the measInterval popup elements, as defined in
  // the Perl code definition of measIntervalValues.  
  if ( service == 'Binex'                  // Binex
    && f.smoothing.checked                 // And smoothing
    && ( f.measInterval.selectedIndex == 1 // 0.1
      || f.measInterval.selectedIndex == 2 // 0.2
      || f.measInterval.selectedIndex == 3 // 0.5
       )
     )
  {
    alert( "Can't smooth carrier on high rate measurements." ) ;
    return false ;
  }

  // If service type is RT17 and Carrier smoothing is checked and
  // measInterval values is 0.1 or 0.2 or 0.5 then alert.
  // Note that this code assumes (for ease of coding) a specific
  // order to the measInterval popup elements, as defined in
  // the Perl code definition of measIntervalValues.  Note that
  // RT17 doesn't have an 'off' setting, so the indices are different
  // from those in binex above.
  if ( service == 'RT17'                   // RT17
    && f.Rt17smoothCarrier.checked         // And carrier smoothing
    && ( f.measInterval.selectedIndex == 0 // 0.1
      || f.measInterval.selectedIndex == 1 // 0.2
      || f.measInterval.selectedIndex == 2 // 0.5
       )
     )
  {
    alert( "Can't smooth carrier on high rate measurements." ) ;
    return false ;
  }

  parent.submitForm( document.theForm, 
                     '/perl-scripts/gpsadmin/setPort.cgi'
                   ) ;
  return false ;

} /* end of Okay() */


/**********************************************************************/
function DeletePort()
/**********************************************************************/
/* Invoked when the Delete button is pressed.
 * We want to delete the setup for the currently displayed
 * TCP/UDP/REP port.  
 * We create a url with the correct Port name as a CGI parameter,
 * and submit the request.  We use loadDataFrame to cause the page
 * to blank prior to submission.  This gives visual feedback and
 * prevents double-submissions, etc.
 */
{
  e = document.theForm.thePort ;
  p = e.options[e.selectedIndex].value ;

  url = '/perl-scripts/gpsadmin/deletePort.cgi?thePort=' + escape( p ) ;

  parent.loadDataFrame( url ) ;

} /* end of DeletePort() */

</Script>
EOF


########################################################################
# Finally we get to actually create the form, using $thePort and 
# all of the previously determined parameters.
########################################################################

print startform( -name=>'theForm',
                 -onSubmit=>'return Okay()'
               ) ;

# Create the port selection popup menu button. 
print popup_menu ( -name=>'thePort',
                   -values=>\@portList ,
                   -default=>$thePort,
                   -onChange=>'restart() ;',
                   -labels=>\%portLabels, 
                 ) ;

# If this is a new port definition, put up the textfield to allow
# entry of the port number.
if ( ($thePort eq 'New TCP Port')
  || ($thePort eq 'New UDP Port')
  || ($thePort eq 'New REP Port')
   )
{
  print "&nbsp;&nbsp;&nbsp;Port Number: ", 
        textfield( 'newPort', '', 5, 5 ) ;
}


# For UDP ports we need to put up a Timeout selector, with a possible
# range of values.  The Default is set to 10 Seconds, assuming it hasn't
# already been set above by reading in an existing configuration.
if ($thePort =~ /udp/i)
{
  print "&nbsp;&nbsp;&nbsp;Timeout: ",
        popup_menu ( -name=>'UDPtimeout',
                     -values=>[ "1 Second",
                                "5 Seconds",
                                "10 Seconds",
                                "30 Seconds",
                                "60 Seconds",
                              ],
                     -default=>"10 Seconds",
                   ) ;
}


if ($thePort !~ /Serial/)
{
  print br(), "&nbsp;&nbsp;&nbsp;&nbsp;",
        checkbox( -name=>'Authenticate',
                  -value=>'1',
                  -label=>'Require Client Authentication',
                ) ;
}

print hr() ;


########################################################################
# The Service
# The range of services available is different for TCP ports versus
# UDP and serial ports.  Create an array of the appropriate types.
########################################################################
my @services ;
if ($thePort =~ /Tcp/i)
{
  @services= ( 'RT17', 'Binex', 'Ephemerides', 'Almanacs', 'SV Status', 
               'RTCM', 'CMR', 'Trimcom'
             ) ;
}
elsif ($thePort =~ /Serial/i)
{
  @services= ( 'PPP', 'RT17', 'Binex', 'Met-Tilt', 'RTCM', 'CMR', 
             ) ;
  # If this is serial port 2 or 4, then add the pppdialout service.
  splice( @services, 1, 0, ('PPP-Dialout') ) 
    if ($thePort =~ /[24]/) ;
}
elsif ($thePort =~ /udp/i)
{
  @services= ( 'RT17', 'Binex', 'RTCM', 'CMR',
             ) ;
}
else
{
  # REP supported services:
  @services= ( 'Binex'
             ) ;
}


my @serviceButtons = radio_group( -name=>'service', 
                            -value=>\@services,
                            -onClick=>'Update() ;'
                          ) ;

print '<b>Service:</b>' ;

print '<Table border="0">' ;
print '<Tr>' ;
for (my $i = 0; $i < @serviceButtons ; $i += 2)
{
  print td("$serviceButtons[$i]"),td("&nbsp;&nbsp;") ;
}

print '<Tr>' ;
for ( my $i = 1; $i < @serviceButtons ; $i += 2)
{
  print td("$serviceButtons[$i]"),td() ;
}

print '</Table>' ;



# Create some popup selectors for use in several types of services.
# These are saved in variables and then dropped into the form
# where needed later on.

my @measIntervalValues = ( 0,
                           0.1 ,    # 0.1 must be index == 1
                           0.2 ,    # 0.2 must be index == 2
                           0.5 ,    # 0.5 must be index == 3
                                    # Modify Okay() function if
                                    # this changes.
                           1   ,
                           2   ,
                           5   ,
                           10  ,
                           15  ,
                           30  ,
                           60  ,
                           300 ,
                           600 ,
                         ) ;

my %measIntervalLabels = ( 0   => 'Off',
                           0.1 => '0.1 Sec', 
                           0.2 => '0.2 Sec', 
                           0.5 => '0.5 Sec', 
                           1   => '1 Sec',
                           2   => '2 Sec',
                           5   => '5 Sec',
                           10  => '10 Sec',
                           15  => '15 Sec',
                           30  => '30 Sec',
                           60  => '1 Min',
                           300 => '5 Min',
                           600 => '10 Min',
                         ) ;

########################################################################
# The Serial Port parameters
########################################################################
# If this is a serial port, put up the Baud/Format/Handshake fields
# The exact combination of options is dependent on which port, and
# which service is assigned to that port.
if ( $thePort =~ /SerialPort/ )
{
  print "<Table Border=0><TR>" ;

  my @bauds = qw/
  2400
  4800
  9600
  19200
  38400
  57600
  115200
  / ;

  # We define the radio_group name to be different between ppp and
  # non-ppp services.  This allows us to switch to the appropriate
  # default when switching the service between ppp and non-ppp.
  my $baudControlName = ($theService ne 'PPP') 
                       ? 'baud'
                       : 'pppbaud' ;
  my $baudDefault = ( ($theService ne 'PPP') 
                      ? $allPortData{$thePort}{'baud'} 
                      : $allPortData{$thePort}{'pppbaud'}
                    ) 
                   || '115200' ;

  print td( b("Baud: "),
            popup_menu ( -name=>$baudControlName,
                         -values=>\@bauds ,
                         -default=>$baudDefault,
                       ) 
          ) ;

  # If this isn't a PPP port we put up the controls to set the parity
  # on the port.
  if ( $theService !~ /^PPP/)
  {
    print td( "&nbsp;&nbsp;".b("Parity:") ) ;

    my $parityDefault = $allPortData{$thePort}{'parity'} 
                     || 'None' ;

    print "Parity Default:$parityDefault<br />" ;
    print td( join( "<br />", 
                    radio_group( -name=>'parity', 
                                 -value=>['None', 
                                          'Even', 
                                          'Odd'
                                         ],
                                 -default=>$parityDefault,
                               )
                  )
            ) ;
  }

  # If this isn't serial port 1, we put up the controls to set the
  # flow control.  The range of values and the defaults for flow 
  # depend on whether the service is ppp or not.  
  # Also, if we are providing PPP-Dialout service on the port, don't
  # put up ANY flow control.  That's all under Dialout options control.
  if ( $thePort ne 'SerialPort1' 
    && $theService ne 'PPP-Dialout'
     )
  {
    my @fcTypes = ('None', 'Hardware' ) ;
    push( @fcTypes, 'Xon/Xoff') if ($theService !~ /^PPP/) ;

    # We define the radio_group name to be different in the two cases.
    # This allows us to switch to the appropriate default when switching
    # the service between ppp and non-ppp.
    my $flowControlName = ($theService eq 'PPP') 
                       ? 'pppflow'
                       : 'flow' ;

    my $flowDefault = ( ($theService eq 'PPP') 
                      ? $allPortData{$thePort}{'pppflow'}
                      : $allPortData{$thePort}{'flow'} 
                    ) 
                   || 'None' ;
    $flowDefault = 'Xon/Xoff' if ($flowDefault eq 'XonXoff' ) ;

    print td( "&nbsp;&nbsp;&nbsp;&nbsp;" ) ;
    print td( b("Flow<br />Control:") ) ;
    print td( join( "<br />", 
                    radio_group( -name=>$flowControlName, 
                                 -value=>\@fcTypes,
                                 -default=>$flowDefault
                               )
                  )
            ) ;
  }

  print "</tr></Table>" ;

} # Serial Port parameters



if ( $theService eq 'Ephemerides'
     or $theService eq 'Almanacs'
     or $theService eq 'SV Status'
     or $theService eq 'Trimcom'
   )
{
  # These services have no parameters.
}
elsif ($theService eq 'CMR')
{
  my $style = $allPortData{$thePort}{'cmrStyle'} ;
  $style = (!$style || ($style eq 'Standard'))
           ? 'standard' 
           : 'cmrplus';
  print '<hr /><br /><b>Parameters for this CMR stream:</b><br />' ;
  my @styleButtons = radio_group( -name=>'cmrStyle', 
                           -value=>[ 'standard', 
                                     'cmrplus', 
                                   ],
                           -nolabels=>1,
                           -default=>$style,
                          ) ;
  print $styleButtons[0],
        "Standard&nbsp;&nbsp;&nbsp;&nbsp;",
        $styleButtons[1],
        "CMR-Plus",
        ;

  # Show the basic reference station setup parameters.
  showRefpos( 'cmr' ) ;
}
elsif ($theService eq 'RTCM')
{

  print '<hr /><br /><b>Parameters for this RTCM stream:</b><br />' ;
  
  my $defaultVersion = 'v'.($allPortData{$thePort}{'version'} || '2.3');
  param( 'rtcmVersion', $defaultVersion ) if (!param('rtcmVersion')) ;

  print br(),
        b('RTCM Format:&nbsp;&nbsp;'),
        popup_menu( -name=>'rtcmVersion',
                    -values=>['v2.1', 'v2.2', 'v2.3', 'v3.0'] ,
                    -default=>$defaultVersion,
                    -onChange=>'Update() ;'
                  ),
        br() 
        ;

  my $rtcmVersion = param( 'rtcmVersion' ) ;


  # Set up a set of buttons allowing the RTCM style selection.
  # Set a default for this parameter if needed.
  # The values from %allPortData are of the forms:
  #   DGPS  RTK  DGPS_RTK  Custom:1,2,3...
  my $defaultStyle = $allPortData{$thePort}{'rtcmStyle'} || 'DGPS' ;

  my $types = $defaultStyle ; # Save any Custom: types.
  $defaultStyle=~ s/:.*// ;   # Drop excess after "Custom:", if any.

  # Set some CGI parameters if not already set.
  if (!param('rtcmStyle')) 
  {
    param( 'rtcmStyle', $defaultStyle ) ;

    if ($defaultStyle eq 'Custom')
    {
      # $types will have Custom:1,2,... containing a comma separated
      # list of types that are already checked.
      # Extract the list, and make sure that all of the fields are
      # surrounded by commas.
      $types =~ s/^Custom:(.*)$/,$1,/ ;
      param( 'Type1' , 1 )    if ( $types =~ /,1,/ ) ;
      param( 'Type2' , 1 )    if ( $types =~ /,2,/ ) ;
      param( 'Type3' , 1 )    if ( $types =~ /,3,/ ) ;
      param( 'Type9' , 1 )    if ( $types =~ /,9,/ ) ;
      param( 'Type14', 1 )    if ( $types =~ /,14,/ ) ;
      param( 'Type1819', 1 )  if ( $types =~ /,18\+19,/ ) ;
      param( 'Type22', 1 )    if ( $types =~ /,22,/ ) ;
      param( 'Type2324', 1 )  if ( $types =~ /,23\+24,/ ) ;
      param( 'Type59.3', 1 )  if ( $types =~ /,59\.3,/ ) ;
      param( 'Type59.13', 1 ) if ( $types =~ /,59\.13,/ ) ;
    }
  }

  my @modeButtons = radio_group( -name=>'rtcmStyle', 
                           -value=>[ 'DGPS', 
                                     'RTK', 
                                     'DGPS_RTK', 
                                     'Custom' 
                                   ],
                           -default=>$defaultStyle,
                           -labels=>{'DGPS_RTK'=>'DGPS+RTK',
                                    },
                           -onClick=>'Update() ;'
                          ) ;

  if ( $rtcmVersion ne 'v3.0')
  {

    print b('Style:&nbsp;&nbsp;'),
          $modeButtons[0], 
          '&nbsp;&nbsp;&nbsp;&nbsp;',
          $modeButtons[1], 
          '&nbsp;&nbsp;&nbsp;&nbsp;',
          $modeButtons[2], 
          '&nbsp;&nbsp;&nbsp;&nbsp;',
          $modeButtons[3], 
          
          br() ;

    if ( param('rtcmStyle') eq 'Custom' )
    {
      print b( 'Messages:' ),
            table( Tr( 
                     td( checkbox( -name=>'Type1',
                                   -value=>'1',
                                   -label=>'Type 1',
                                   -onClick=>'Type1Click()'
                                 )
                       ),
                     td( checkbox( -name=>'Type2',
                                   -value=>'1',
                                   -label=>'Type 2',
                                 )
                       ),
                     td( checkbox( -name=>'Type3',
                                   -value=>'1',
                                   -label=>'Type 3',
                                 )
                       ),
                     # Type 9 control has been removed (SPR #167) until
                     # such time as we actually implement it.
                     #td( checkbox( -name=>'Type9',
                     #              -value=>'1',
                     #              -label=>'Type 9',
                     #              -onClick=>'Type9Click()'
                     #            )
                     #  ),
                     td( checkbox( -name=>'Type1819',
                                   -value=>'1',
                                   -label=>'Types 18+19',
                                 )
                       ),
                     ),

                   Tr( 
                     td( checkbox( -name=>'Type22',
                                   -value=>'1',
                                   -label=>'Type 22',
                                 )
                       ),

                     ($rtcmVersion ne 'v2.1' and $rtcmVersion ne 'v2.2')
                     ? td( checkbox( -name=>'Type2324',
                                     -value=>'1',
                                     -label=>'Types 23+24',
                                   )
                         )
                     : ''
                     ,

                     td( checkbox( -name=>'Type59.3',
                                   -value=>'1',
                                   -label=>'Type 59.3',
                                 )
                       ),
                     td( checkbox( -name=>'Type59.13',
                                   -value=>'1',
                                   -label=>'Type 59.13',
                                 )
                       ),
                     )
                 )
            ;

      # Need a few javascript functions to handle some mutually 
      # exclusive items in the Custom buttons.  Type1 and Type9
      # are not allowed together.  If either is clicked, the appropriate
      # function below is called, which turns off the other one.
      print <<EOF ;
      <script>
      /****************************************************************/
      function Type1Click()
      /****************************************************************/
      /* Called whenever the Type1 checkbox is clicked, either when
       * checked or unchecked.  If checked, then we force the Type9
       * checkbox to be unchecked.
       */
      {
        f=document.theForm ;
        if (f.Type1.checked)
          f.Type9.checked = false ;
      }

      /****************************************************************/
      function Type9Click()
      /****************************************************************/
      /* Called whenever the Type9 checkbox is clicked, either when
       * checked or unchecked.  If checked, then we force the Type1
       * checkbox to be unchecked.
       */
      {
        f=document.theForm ;
        if (f.Type9.checked)
          f.Type1.checked = false ;
      }

      </script>
EOF
    }
  }

  # Show the basic reference station setup parameters.
  showRefpos( 'rtcm' ) ;

}
elsif ($theService eq 'Rinex')
{
  print '<br />Undefined as of now.' ;
  print '<br />Not even sure if this is desired.' ;
}
elsif ($theService eq 'Binex')
{
  print '<hr /><br /><b>Parameters for this Binex stream:</b><br />' ;

  # Determine the base meaurement rate.  If undefined, then set it
  # to 1.0 seconds.
  my $measInterval = $allPortData{$thePort}{'interval'} || 1 ;

  # If this port wasn't an existing Binex port, then we need to
  # set up a few default values.  All of the Binex specific
  # hash elements should be undefined, which should cause their
  # associated checkboxes to be unchecked.  So we only have to
  # specify those we want checked by default.
  if ( !defined( $allPortData{$thePort}{'7f03'} ) )
  {
    $allPortData{$thePort}{'7f03'} = 1 ;
    $allPortData{$thePort}{'smoothing'} = 1 ;
    $allPortData{$thePort}{'metTilt'} = 1 ;
  }

  my @metaIntervalValues = ( 0,
                             60,
                             300 ,
                             900 ,
                             1800,
                             3600
                           ) ;

  my %metaIntervalLabels = ( 0   => 'Off',
                             60  => '1 Min',
                             300 => '5 Min',
                             900 => '15 Min',
                             1800 => '30 Min',
                             3600 => '60 Min',
                           ) ;

  print "<br /><table width=\"100%\">\n" ;
  print "<tr>" ;
  print td( table( 
                Tr( th( {colspan=>2}, "Measurements" ) ),
                Tr( 
                 td( {colspan=>2, align=>'center'},
                     popup_menu ( -name=>'measInterval',
                                  -values=>\@measIntervalValues,
                                  -labels=>\%measIntervalLabels,
                                  -default=>$measInterval,
                                ),
                     '&nbsp;&nbsp;Interval'
                   ),

                  ),
                Tr( 
                 td( checkbox( -name=>'smoothing',
                               -value=>'1',
                               -label=>'',
                           -checked=>$allPortData{$thePort}{'smoothing'}
                             ), 
                   ),
                 td( 'Smooth Carrier Phase' 
                   ),
                  ),
                Tr(
                 td( checkbox( -name=>'7f02',
                               -value=>'1',
                               -label=>'',
                         -checked=>$allPortData{$thePort}{'7f02'}
                             ), 
                   ),
                 td( 'Send 7f-02 Measurements' 
                   )
                  ),
                Tr(
                 td( checkbox( -name=>'7f03',
                               -value=>'1',
                               -label=>'',
                         -checked=>$allPortData{$thePort}{'7f03'}
                              ), 
                   ),
                 td( 'Send 7f-03 Measurements' 
                   )
                  ),
                Tr(
                 td(),
                 td( checkbox( -name=>'7f03Doppler',
                               -value=>'1',
                               -label=>'',
                        -checked=>$allPortData{$thePort}{'7f03Doppler'}
                             ), 
                     "With L1 Doppler"
                   ),
                  ),
                 Tr(
                 td(),
                 td( checkbox( -name=>'7f03Slips',
                               -value=>'1',
                               -label=>'',
                        -checked=>$allPortData{$thePort}{'7f03Slips'}
                             ), 
                     "With Cycle Slip Counters"
                   ),
                  ),
                Tr(
                 td(),
                 td( checkbox( -name=>'Always',
                               -value=>'1',
                               -label=>'',
                 -checked=>$allPortData{$thePort}{'7f03RxClkOfsAlways'},
                               -onClick=>'alwaysClicked();',
                             ), 
                     "With Clock Offsets - Always"
                   ),
                  ),
                Tr(
                 td(),
                 td( checkbox( -name=>'Rollover',
                               -value=>'1',
                               -label=>'',
               -checked=>$allPortData{$thePort}{'7f03RxClkOfsRollover'},
                               -onClick=>'rolloverClicked();',
                             ), 
                     "With Clock Offsets - On Rollovers"
                   ),
                  ),
                Tr( td( '&nbsp;' ) ), 
                Tr( th( {colspan=>2}, "System Status Record" ) ), 
                Tr( td( {align=>'center', colspan=>2},
                        popup_menu ( -name=>'statusInterval',
                                     -values=>\@metaIntervalValues,
                                     -labels=>\%metaIntervalLabels,
                      -default=>$allPortData{$thePort}{'statusInterval'}
                                   ),
                        "&nbsp;&nbsp;Interval"
                      ),
                  ) 
                 )
          ) ;

#  print td( {width=>30} ) ;

  print td( {valign=>'top'},
            table( 
                   Tr( th( "MetaData" ) ),
                   Tr( td( {align=>'center'},
                            popup_menu ( -name=>'metaInterval',
                                         -values=>\@metaIntervalValues,
                                         -labels=>\%metaIntervalLabels,
                        -default=>$allPortData{$thePort}{'metaInterval'}
                                       ),
                            '&nbsp;&nbsp;Interval'
                          ),
                      ),
                    Tr( 
                        td( checkbox( -name=>'siteDesc',
                                      -value=>'1',
                                      -label=>'Site Description',
                            -checked=>$allPortData{$thePort}{'siteDesc'}
                                    )
                          ),
                      ),
                    Tr( 
                       td( checkbox( -name=>'stationID',
                                     -value=>'1',
                                     -label=>'Station ID',
                           -checked=>$allPortData{$thePort}{'stationID'}
                                   )
                         ),
                     ),
                   Tr( 
                       td( checkbox( -name=>'rcvrType',
                                     -value=>'1',
                                     -label=>'Receiver Type',
                            -checked=>$allPortData{$thePort}{'rcvrType'}
                                   )
                         ),
                     ),
                   Tr( 
                       td( checkbox( -name=>'rcvrNumber',
                                     -value=>'1',
                                     -label=>'Rinex Receiver Number',
                          -checked=>$allPortData{$thePort}{'rcvrNumber'}
                                   )
                         ),
                     ),
                    Tr(
                        td( checkbox( -name=>'firmwareVersion',
                                      -value=>'1',
                                      -label=>'Firmware Version',
                     -checked=>$allPortData{$thePort}{'firmwareVersion'}
                                    )
                          ),
                      ),
                    Tr(
                       td( checkbox( -name=>'antXYZ',
                                     -value=>'1',
                                     -label=>'Antenna XYZ',
                              -checked=>$allPortData{$thePort}{'antXYZ'}
                                   )
                         ),
                      ),
                    Tr(
                       td( checkbox( -name=>'antOffset',
                                     -value=>'1',
                                     -label=>'Antenna Offset',
                           -checked=>$allPortData{$thePort}{'antOffset'}
                                   )
                         ),
                      ),
                    Tr(
                       td( checkbox( -name=>'antNum',
                                     -value=>'1',
                                     -label=>'Rinex Antenna Number',
                              -checked=>$allPortData{$thePort}{'antNum'}
                                   )
                         ),
                      ),
                    Tr( td( {height=>"5"} ) ), 
                    Tr( th( "Met/Tilt Data" ) ),
                    Tr( td( checkbox( -name=>'metTilt',
                                      -value=>'1',
                                      -label=>'Include in Stream',
                             -checked=>$allPortData{$thePort}{'metTilt'}
                                    )
                          ),
                      )
                 )
          ) ;

  print "</tr>" ;

  print "\n</table>\n" ;

  print <<EOF ;
  <script>
  /********************************************************************/
  function alwaysClicked()
  /********************************************************************/
  /* Called when the Binex With Clock Offset Always checkbox is clicked.
   * If the checkbox is active, then we deselect the On Rollovers
   * checkbox.
   */
  {
    f = document.theForm ;
    if (f.Always.checked)
    {
      f.Rollover.checked = false ;
    }
  } // End of alwaysClicked()

  /********************************************************************/
  function rolloverClicked()
  /********************************************************************/
  /* Called when the Binex WIth Clock Offset Rollover checkbox is
   * clicked.  If the checkbox is active, then we deselect the Always
   * checkbox.
   */
  {
    f = document.theForm ;
    if (f.Rollover.checked)
    {
      f.Always.checked = false ;
    }
  } // End of rolloverClicked()

  </script>
EOF

}
elsif ($theService eq 'Met-Tilt')
{
  print '<hr /><br /><b>Parameters for Met-Tilt mode:</b><br />' ;

  print '<Table border="1">' ;
  print Tr( th( ["&nbsp;Command&nbsp;", "&nbsp;Schedule&nbsp;"] ) ) ;

  my @values = ( "Once", 
                 "1 second",
                 "5 seconds",
                 "15 seconds",
                 "30 seconds",
                 "1 minute", 
                 "5 minutes", 
                 "10 minutes", 
                 "15 minutes", 
                 "30 minutes", 
                 "1 hour", 
                 "2 hours", 
                 "4 hours", 
                 "Off"
               ) ;
  for my $i (1..6) 
  {
    my ( $sched, $cmd) = 
                     split( ":", $allPortData{'MetControls'}{$i}) ;

    if ( ! defined($cmd) )
    {
      $sched = 'Off' ;
    }
    elsif ($sched eq 'once' )
    {
      $sched = 'Once' ;
    }
    elsif ($sched eq '1S')
    {
      $sched = '1 second' ;
    }
    elsif ($sched eq '5S')
    {
      $sched = '5 seconds' ;
    }
    elsif ($sched eq '15S')
    {
      $sched = '15 seconds' ;
    }
    elsif ($sched eq '30S')
    {
      $sched = '30 seconds' ;
    }
    elsif ($sched == 0 )
    {
      $sched = 'Once' ;
    }
    elsif ($sched == 60)
    {
      $sched = '1 hour' ;
    }
    elsif ($sched > 60)
    {
      $sched = ($sched/60)." hours" ;
    }
    elsif ($sched == 1)
    {
      $sched = '1 minute' ;
    }
    else
    {
      $sched = $sched." minutes" ;
    }
    param( "Sched$i", $sched ) ;
    print Tr( td( textfield( "Cmd$i", $cmd, 30, 60 ) ) ,
              td( popup_menu ( -name=>"Sched$i",
                                      -values=>\@values,
                                      -default=>$sched
                                    ) 
                       ) ,
            ) ;
  }
  print '</Table>' ;
  
 
}
elsif ($theService eq 'PPP')
{
  print '<hr /><br /><b>Parameters for PPP mode:</b><br />' ;
  print '<table>' ;
  print Tr( td( {align=>'right'}, 'Local IP Address' ), 
            td( textfield( 'localIP', 
                           $allPortData{$thePort}{'localip'},
                           15, 
                           15 
                         ) 
              ) 
          ) ;
  print Tr( td( {align=>'right'}, 'Remote IP Address' ), 
            td( textfield( 'remoteIP', 
                           $allPortData{$thePort}{'remoteip'},
                           15, 
                           15 
                         ) 
              )
          ) ;

  param( 'connection', $allPortData{$thePort}{'connection'} ) 
    if (!param( 'connection' ) ) ;

  print Tr( td( {align=>'right'}, 'Connection' ),
            td( radio_group( -name=>'connection', 
                             -value=>['Direct','Modem'],
                             -onClick=>'Update() ;'
                           )
              )
          ) ;

  if (param('connection') eq 'Modem')
  {
    print Tr( td( {align=>'right'}, 'Init Strings' ),
              td( textfield( 'initString1', 
                             $allPortData{$thePort}{'init1'}, 
                             15, 
                             15 
                           ) 
                )
            ) ;

    print Tr( td( '&nbsp;' ),
              td( textfield( 'initString2', 
                             $allPortData{$thePort}{'init2'}, 
                             15, 
                             15 
                           ) 
                )
            ) ;

    print Tr( td( '&nbsp;' ),
              td( textfield( 'initString3',
                             $allPortData{$thePort}{'init3'}, 
                             15, 
                             15 
                           ) 
                )
            ) ;

    print Tr( td( {align=>'right'}, 'Reset String' ),
              td( textfield( 'resetString', 
                             $allPortData{$thePort}{'reset'}, 
                             15, 
                             15 
                           )
                )
            ) ;
  }

  print '</table>' ;

}
elsif ($theService eq 'PPP-Dialout')
{
  print hr(), br(),
        '<b>Parameters for this PPP-Dialout Stream:</b>', 
        br(), br(),
        'Use this page to assign PPP Dialout to a ',
        'specific port and set the Baud Rate.', 
        '<br />Other parameters for PPP Dialout are specifed on the ',
        a( {href=> 'javascript:parent.loadDataFrame'
                  .'("/perl-scripts/sysadmin/pppDialout.cgi")'
           },
           'PPP Dialout'
         ) , 
        ' page.',
        br(),
        br() ;

}
elsif ($theService eq 'RT17')
{
  print '<hr /><br /><b>Parameters for this RT17 Stream:</b><br />' ;

  print '<Table border=1><Tr><Td valign="top">' ;

  # If we are showing RT17 controls, but the RT17 settings haven't been
  # defined, as when creating a new port or switching between services,
  # Set up some defaults.  Those boolean controls not mentioned will
  # be undefined and hence appear as unchecked below.
  if (! defined( $allPortData{$thePort}{'concise'} ) )
  {
    $allPortData{$thePort}{'measurements'} = 1 ;
    $allPortData{$thePort}{'concise'} = 1 ;
    $allPortData{$thePort}{'gpsdata'} = 1 ;
    $allPortData{$thePort}{'iodeslips'} = 1 ;
  }


  #my $measDefault = $allPortData{$thePort}{'measurements'} ;
  #$measDefault = 1 if ( ! $allPortData{$thePort}{'positions'} ) ;

  # Drop the OFF setting for RT17 streams.
  shift( @measIntervalValues ) ; 

  print table( Tr(th( 'Epoch Interval' ) ), 
               Tr(td( {align=>'center'}, 
                      popup_menu ( -name=>'measInterval',
                                   -values=>\@measIntervalValues,
                                   -labels=>\%measIntervalLabels,
                                   -default=>
                                      $allPortData{$thePort}{'interval'}
                                      || 1
                      ) 
                    ) 
                 ),
               Tr(td( checkbox( -name=>'Rt17Positions',
                                -value=>'1',
                                -label=>'',
                                -onClick=>'cbClick();',
                                -checked=> 
                                     $allPortData{$thePort}{'positions'}
                              ),
                      "Positions"
                    ) 
                 ),
               Tr(td( checkbox( -name=>'Rt17Measurements',
                                -value=>1,
                                -label=>'',
                                -onClick=>'cbClick();',
                                -checked=>
                                  $allPortData{$thePort}{'measurements'}
                              ),
                      "Measurements",
                    ) 
                 ),
             ) ;
  print <<EOF ;
<script>
function cbClick()
{
  f = document.theForm ;

  //alert( f.Rt17Positions.checked + " " + f.Rt17Measurements.checked ) ;
  if ( ! f.Rt17Positions.checked && ! f.Rt17Measurements.checked )
  {
    f.Rt17Measurements.checked = true ;
  }
}
</script>
EOF

  print '</td><td valign="top">' ;
  print table( Tr(th( {colspan=>2}, 'Options' ) ),

               Tr(td( [checkbox( -name=>'Rt17Concise',
                                -value=>'1',
                                -label=>'',
                                -checked=>
                                       $allPortData{$thePort}{'concise'}
                              ),
                      "Use Concise Format" ]
                    ) 
                 ),
               Tr(td( [checkbox( -name=>'Rt17Raw',
                                -value=>'1',
                                -label=>'',
                                -checked=>
                                        $allPortData{$thePort}{'rawgps'}
                              ),
                      "Send Raw GPS Data" ]
                    ) 
                 ),
               Tr(td( [checkbox( -name=>'Rt17Waas',
                                -value=>'1',
                                -label=>'',
                                -checked=>
                                       $allPortData{$thePort}{'rawwaas'}
                              ),
                      "Send Raw WAAS Data" ]
                    ) 
                 ),
               Tr(td( [checkbox( -name=>'Rt17svinfo',
                                -value=>'1',
                                -label=>'',
                                -checked=>
                                       $allPortData{$thePort}{'gpsdata'}
                              ),
                      "Send Ephemeris GPS Data" ]
                    ) 
                 ),
               Tr(td( [checkbox( -name=>'Rt17extras',
                                -value=>'1',
                                -label=>'',
                                -checked=> 
                                     $allPortData{$thePort}{'iodeslips'}
                              ),
                      "Send IODE values and<br />Cycleslip Counts" ]
                    ) 
                 ),
               Tr(td( [checkbox( -name=>'Rt17smoothCode',
                                -value=>'1',
                                -label=>'',
                                -checked=>
                                    $allPortData{$thePort}{'smoothcode'}
                              ),
                      "Smooth Code Phase" ]
                    ) 
                 ),
               Tr(td( [checkbox( -name=>'Rt17smoothCarrier',
                                -value=>'1',
                                -label=>'',
                                -checked=>
                                 $allPortData{$thePort}{'smoothcarrier'}
                              ),
                      "Smooth Carrier Phase" ]
                    ) 
                 ),
               Tr(td( [checkbox( -name=>'Rt17useRt27',
                                 -value=>'1',
                                 -label=>'',
                                 -checked=>
                                  $allPortData{$thePort}{'useRt27'}
                               ),
                      "Use Record Type 27" ]
                    ) 
                 ),
             ) ;

  print "</td></Tr></Table><br />" ;

}
else
{
  print hr(),
        br(),
        b("Options for service $theService are currently Undefined."),
        br(),
        ;
}



########################################################################
# Form control buttons - OK, Cancel and Delete.
########################################################################
print hr() ;

print button( -name=>'OK', 
              -value=>'OK',
              -onClick=>'Okay() ;',
            ) ;
print '&nbsp;' ;
print button( -name=>'Cancel', 
              -value=>'Cancel',
              -onClick=>'parent.loadDataFrame( "portSummary.cgi" )'
            ) ;

# If this form is editing a user created TCP/UDP/REP port setup, then
# give the opportunity to delete that setup.
if ($thePort !~ /^(Serial|New)/)
{
  print '&nbsp;' ;
  print button( -name=>'Delete', 
                -value=>'Delete',
                -onClick=>'DeletePort() ;'
              ) ;
}

print endform() ;

print end_html() ;


########################################################################
# End of file portSetup.cgi
########################################################################
