#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: sessions.cgi,v 1.52 2004/10/15 22:04:44 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/sessions.cgi,v $

########################################################################
# sessions.cgi
#
# Script for displaying a summary of the current data logging sessions
# in fireball.  Contains buttons for invoking screens for creating
# new sessions and editing existing ones.
#
# This is an auto-refreshing screen because it shows realtime status of
# the system.  A configuration file /etc/sysconfig/sessions contains the
# descriptions of all data logging sessions.  This is read and parsed
# into various hashes.  That data is combined with runtime information
# to determine which sessions are active, pending, disabled, etc.
########################################################################

use strict ;
use CGI qw/:standard/;
use Fireball ;

titleWithHelp( 'Data Logging Status',
               '/sessionsInfo.html'
             ) ;

# Get the time of day in GPS seconds.  Includes a 13 second offset for
# the UTC to GPS leapseconds.  315964800 converts from Unix time to
# gps time (01Jan1970 to 06Jan1980).
my $gpsTime = time() - 315964800 + 13 ;


# Read in the data from the sessions configuration file.
my %sessData ;
getSessionData( \%sessData ) ;


# Read in the descriptions of any active sessions, as defined by
# files of the form /var/run/gpslog.*.  @ActiveSessions will contain
# an array of the contents of these files, which can be searched for
# specific session names, and the parameters of the active session(s)
# associated with those names.
my @ActiveSessions = () ;
for my $f (glob '/var/run/gpslog.*')
{
  # Open and read that file.
  # Note that though the file $f existed above (glob found it!) it
  # may have gone away before we got around to reading it.  So test
  # if the open succeeds here:
  if ( open( LOG,  $f ) )
  {
    my $line = <LOG> ; # Get first line

    # The file should always have a line, but sometimes it doesn't
    # during testing.  If not, it will be empty and we'll ignore it.
    if ($line)
    {
      # Add to list of sessions if the file contained anything
      push @ActiveSessions, $line ;
    }
    close LOG ;
  }
}


print startform() ;

# All of the items displayed on this page are center aligned on the
# page, so we define a 'div' section to cover them all.
print '<div align="center">' ;


# In the main table, we want it to be tight spacing, to keep the 
# width down.  But on many browsers, the left/right padding on each cell
# is too tight.  So we define some style overrides for the cells.
# These can be applied to individual cells below using "class=tight".
# Note that we shrink the size of text in the column headers to make
# them fit better.
print <<END ;
<style>
<!--
td.tight {padding-left: 5px;  padding-right: 5px }
th.tight {padding-left: 5px;  padding-right: 5px ; font-size:smaller}
-->
</style>
END


print '<p><Table border>' ;
print Tr( th( {class=>"tight"}, 'Session<br />Name' ),
          th( {class=>"tight"}, 'Schedule' ),
          th( {class=>"tight"}, 
              'Duration<br /><font size="-2">[minutes]</font>' 
            ),
          th( {class=>"tight"}, 'Status' ),
          th( {class=>"tight"}, 'Enable' )
        ) ;

foreach my $sessname (sort keys( %sessData))
{
  my $enable = $sessData{$sessname}{'enable'} ;
  my $sched = $sessData{$sessname}{'sched'} ;
  my $time = $sessData{$sessname}{'time'} ;
  my $duration = $sessData{$sessname}{'duration'} ;

  print "<tr align='center'>" ;

  my $link= "javascript:parent.loadDataFrame('editSession.cgi?restart="
          .CGI::escape($sessname)
          ."')" ;
  print td( {class=>"tight"}, a( {href=>$link}, $sessname) ) ;

  if ($sched eq "manual") 
  { 
    print td( {class=>"tight"}, 'Manual' ) ;
  }
  elsif ($sched eq "cont") 
  { 
    print td( {class=>"tight"}, "Continuous" ) ;
  }
  elsif ($sched eq "daily") 
  { 
    print td( {class=>"tight"}, 
              sprintf( "Daily at %02d:%02d", 
                       int( $time / 3600 ) ,
                       int( $time % 3600 / 60 ) 
                     )
            ) ;
  }
  else
  {
    # Once only session
    my (undef,$min,$hour,$day,$month,$year,undef)
      = gmtime( $time + 315964800 ) ;

    $month = $monthStrings[$month] ;

    print td( {class=>"tight"},
              sprintf( '%04d/%3s/%02d@%02d:%02d', 
                       $year+1900, $month, $day, $hour, $min
                     )
            ) ;
  }

  print td( $duration ) ;

  # Print the status
  my $status = "" ;

  # This session is enabled, and the scheduler may be running it.
  # Search for it in the active sessions stuff.
  for my $s (@ActiveSessions)
  {
    my (undef, undef, $name, undef, undef, undef, 
        undef, undef, $fpath) = split( ' ', $s ) ;
    if ($sessname eq $name)
    {
      # Split the path into directory and fname, and drop the
      # leading /var/gps.  We hide that info from user.
      my ($dir, $fname) = ($fpath =~ m|^/var/gps(/.*/)(.*)| ) ; 

      # There might be two sessions running on the same name (if
      # continuous logging).  If we found one already, put the two of 
      # them on separate lines.  [Latest continuous logging mode
      # doesn't behave like this, but no harm in leaving this in.]
      $status .= '<br />' if ($status) ;

      $status .= "<font size=-1>logging to $dir</font><br />$fname" ;
    }
  }

  if ( ! $status )
  {
    # Session not actively logging (perhaps, not yet.)
    # Session is either Disabled, Pending or Completed.
    # Continuous sessions are either disabled or always running, But
    #   if they are JUST starting up, we'll indicate they are pending.
    # Daily sessions are disabled or always either running or pending,
    #   so they are always pending if we get to here.  
    # Once-only sessions have a single time window with a
    #   well defined stop-time.  If disabled, we just say so.  If we 
    #   are before that stop time (and since we aren't running) we'll
    #   display pending.  Otherwise we'll display 'completed'.
    # Manual sessions also have a well defined time window, and should
    #   be similar to Once-only sessions.  However, their 'enable' state
    #   is artificially made false if the window has passed, so that the
    #   user can re-trigger the session.  So we infer this condition
    #   to show the 'completed' state.  Note that if a manual session
    #   is actually disabled, using the checkboxes, it's start
    #   time is adjusted to decades in the future, so it will show
    #   Disabled.

    if ( $sched eq 'cont' 
      or $sched eq 'daily'
       )
    {
      $status = $enable ? 'Pending' : 'Disabled' ;
    }
    elsif ( $sched eq 'once' )
    {
      $status = (! $enable) 
                ? 'Disabled'
                : ( $gpsTime >= ($time + $duration*60) )
                  ? 'Completed'
                  : 'Pending' ;
    }
    else
    {
      # manual schedule
      $status = ( $gpsTime >= ($time + $duration*60) ) 
                ? 'Completed'
                : ($enable)
                  ? 'Pending'
                  : 'Disabled' ;
    }
  }
  print td( {class=>"tight"}, $status ) ;

  # If this is a manual session and it has already completed, we 
  # artificially indicate that the session is now disabled.  This lets
  # the checkbox restart the session.
  if ( ($sched eq 'manual') 
    && ($status =~ /Completed/)
     )
  {
    $enable = '0' ;
  }

  my $target= 'sessionEnableDisable( "'
              . $sessname
              . '", "'
              . ($enable ? 'disable")' : 'enable")')
              ;
  print td( {class=>"tight"}, 
            checkbox( -name=>'Enabled',
                      -checked=>$enable,
                      -value=>'1',
                      -label=>'',
                      -onClick=> $target
                    ) 
          ) ;

  print "</tr>" ;
}

print '</Table>' ;
print endform() ;

# Emit the javascript function used to submit the Enable/Disable
# requests above.
print <<EOF ;
<script>

function sessionEnableDisable( sessname, state )
{
  url = '/perl-scripts/gpsadmin/startSessions.cgi?'
      + state 
      + '=' 
      + escape(sessname) ;
  parent.loadDataFrame( url ) ;
}

</script>
EOF

if (! scalar( keys(%sessData) ) ) 
{
  print '<p><b>No sessions are currently defined.</b><p>' ;
}

# Display the PowerSaving mode control.
print br() ;
if ( open( SLEEP, '/etc/sysconfig/sleepmode' ) )
{
  my $line=<SLEEP> ;
  close SLEEP ;
  chomp $line ;
  my ($Enable,$Periodic,$Hours,$Minutes) = split( ' ', $line ) ;

  print a( {href=>"javascript:parent.loadDataFrame('sleepMode.cgi')"}, 
           'Power Saving Mode' 
         ) ;
  if ($Enable eq 'yes')
  {
    print  ' is Enabled.<br />' ;
    if ($Periodic eq 'yes')
    {
      print "Periodic wakeup set to $Minutes minutes every $Hours ",
            $Hours > 1 ? "hours." : "hour." ;
    }
    else
    {
      print 'Periodic wakeup is disabled.' ;
    }
  }
  else
  {
    print ' is Disabled.' ;
  }
}

print br(),br() ;



# Get the diskspace available data.  Sadly, Perl doesn't have an
# easy way to get this via a system call.  We just run the "df" command
# and parse the output, which will look something like this:
#
#   Filesystem           1k-blocks      Used Available Use% Mounted on
#   rootfs                 6174820   3634344   2226808  62% /
#
# This is just split into a list, and the appropriate fields lifted
# from that.  'Filesystem' is field[0], 'on' is [6], etc.
my($usedSpace, $availSpace) = ( split " ", qx( df /var/gps ) )[9..10] ;

my ($journalSize) = qx( sudo /usr/local/fireball/ext3JournalSize ) ;
$usedSpace -= $journalSize ;

# Display the space information.
# If we are below 1Mbyte left (autodelete should kick in) make the
# space-left text bold-Red.  If we are below 100Kbytes - where logging
# won't even be started, also flash the text.  THis gives a visual
# indication of WHY if logging has stopped.
# To implement this we create an HTML string that will be inserted
# before the text to start the emphasis, and a matching string that is
# placed after the text.
my $emphasisStart = '' ;
my $emphasisEnd = '' ;
if ($availSpace < 1024)
{
  $emphasisStart = '<b><font color="red">' ;
  $emphasisEnd = '</font></b>' ;
}
if ($availSpace < 1024)
{
  $emphasisStart .= "<blink>" ;
  $emphasisEnd = "</blink>".$emphasisEnd ;
}

print "<table border=0>" ;

printf Tr( td( {align=>'right'}, "Logging Space:" ),
           td( {class=>'tight', colspan=>5},
               sprintf( "%.3f Mbytes Used.&nbsp;&nbsp;&nbsp;"
                       ."%s%.3f Mbytes Available%s",
                        $usedSpace/1024, 
                        $emphasisStart,
                        $availSpace/1024, 
                        $emphasisEnd
                      )
             ),
         )
       ;


my $link = "javascript:parent.loadDataFrame('autoDelete.cgi')" ;

my %autoDeleteControls ;
getAutoDeleteControls( \%autoDeleteControls ) ;
my $size = $autoDeleteControls{'Reserved'}{'size'} ;

print Tr( 
          td( {align=>'right'}, a( {href=>$link},'Reserved Space:') ),
          td( {class=>'tight', colspan=>5},
              $autoDeleteControls{'Reserved'}{'enable'}
              ?  "AutoDeleting at "
                 . "$size Mbyte"
                 . ($size==1 ? '.' : 's.')
              : '<b>Not</b> AutoDeleting.'
            )
        ) ;


# Display the logging pool status.
my %poolStatus ;
getPoolStatus( \%poolStatus ) ;

if ( keys %poolStatus )
{
  for my $pool ('a'..'z','Default')
  {
    if (exists( $poolStatus{$pool} ) ) 
    {
      my $ad = $poolStatus{$pool}{'ad'} eq 'yes' ; 
      my $size = $poolStatus{$pool}{'size'}/1000000 ;
      my $maxSize = $poolStatus{$pool}{'maxsize'}/1000000 ;

      my $full = ( ( ! $ad ) && ($size >= $maxSize) ) ;

      print Tr(
              td( {align=>'right'}, a( {href=>$link},"$pool-pool:") ),
              td( {class=>'tight', align=>'right'},
                  "$poolStatus{$pool}{'files'} files" 
                ),
              td( {align=>'right'},
                  sprintf( "%.1f Mb /", $size ) 
                ),
              td( {align=>'right'},
                  sprintf( "%.0f Mb", $maxSize ) 
                ),
              td( {class=>'tight', align=>'right'},
                  ($ad ? '' : 'Not ') . "AutoDeleting" 
                ),
              td( $full 
                  ? '<font color="red"><blink> FULL</blink></font>'
                  : '&nbsp;'
                )

              ) ;
    }
  }
}
else
{
  print Tr( td( {colspan=>4, align=>'center'},
                "No data logging pools defined." 
              ) 
          ) ;
}
print "</table>" ;

# Create a one line table of "buttons" for various actions that
# might be desired from the Sessions status page.
# Note that since this page is autorefreshing every few seconds,
# using a timer in the parent window (setup at the script end below)
# you can't just link to a new location.  The timer would kick in and
# overwrite you.  Instead, use the parent function loadDataFrame, which
# cancels the timer.

# One of the buttons switches to editing existing sessions, and picks
# the first already defined session name.  
my $firstSessName = (sort keys %sessData)[0] ;
$firstSessName = "New Session" if (! $firstSessName) ;
$firstSessName = CGI::escape( $firstSessName ) ;

$link= "javascript:parent.loadDataFrame('editSession.cgi?restart="
          .$firstSessName
          ."')" ;

print br(),
      table( {border=>2,align=>'center',cellpadding=>5},
        Tr( 
            td( {align=>'center', width=>"50%" },
                a( {href=>
                    "javascript:parent.loadDataFrame('editSession.cgi')"
                   } , 
                   "Create a New<br />Session" 
                 )
              ),
            td( {align=>'center', width=>"50%"},
                a( {href=>
                    $link 
                   }, 
                   "Edit<br />Sessions" 
                 )
              ),
          )
        ) ;



# Show a Timestamp:
print '<p>',timestamp() ;




# End of section where everything is center aligned.
print '</div>' ;

print <<EOF ;
<script>
  // Reload this page every ten seconds
    parent.loadAfterDelay( window.location.href, 10000) ;
</script>

EOF


print end_html() ;

########################################################################
# end of script sessions.cgi
########################################################################
