#!/usr/bin/perl -w
# Copyright (c) 2003 Trimble Navigation Limited
# $Id: editAccount.cgi,v 1.27 2004/10/19 20:08:54 tom Exp $
# $Source: /home/CVS/fireballTop/fireballFS/root/usr/local/apache/htdocs/fireball/perl-scripts/sysadmin/editAccount.cgi,v $

########################################################################
# editAccount.cgi
#
# Script for creating and processing the forms used for modifying
# new or existing accounts.  
#
# Can be called one of several ways:
#   o With no CGI parameters - 
#     We put up a form for a NEW account
#
#   o With CGI parameters to start editing a specific account.
#       ?restart=accname
#     We put up the same form (as NEW) but with the data for an
#     existing account.
#       
#   o With CGI parameters requesting an account deletion.
#       ?delete=accname
#
#   o With CGI parameters showing that new data has been accepted
#     Possible parameters are:
#       ?account    =>Value of Popup account selector
#                     =="NewAccount" if this is a new accounts setup.
#       ?AccountName=>Text field, New Account name
#       ?sysadmin   =>'yes' if checkbox is clicked.
#       ?GPSadmin   =>'yes' if checkbox is clicked.
#       ?FileAccess =>'yes' if checkbox is clicked.
#       ?Password1  =>Text field
#       ?Password2  =>Text field
#     We check the parameters and react:
#       - All okay, we create the new account or we modify the existing
#         account.
#       - Problem.  We update the form and include a Javascript
#         Alert call that displays an error message.
#
# The information describing the accounts comes from the files
# /etc/sysconfig/htpasswd and /etc/sysconfig/htgroups.  The values from
# those files are read in as the defaults when modifying existing
# accounts.  And when a change is made to any accounts, those files
# are re-created to show the new data.
#
########################################################################



use strict ;
use CGI qw/:standard/ ;

use lib '..' ;
use Fireball ;

titleWithHelp( 'Security - Accounts',
               '/securityAccounts.html'
             ) ;
           

# Read the current GUI account state into the local hash %accounts.
# Keys are the account names.
# Each key defines a sub-hash containing 
#   'password'   : encrypted password
#   'sysadmin'   : boolean indicating System Controls capabilities
#   'gpsadmin'   : boolean indicating GPS Controls capabilities
#   'fileaccess' : boolean indicating File Access capabilities
my %accounts ;
getGuiAccounts( \%accounts ) ;


# Some global variables that may be changed by CGI parameters
my $popupDefault = "New Account" ;

# If this is a CGI request then some parameters will be passed into the
# script.  We'll need to parse those and do what is requested.  This
# process may find errors, the first of which will be saved in $error.
my $error='' ;

if (param())
{
  # cgi parameters were appended.
  
  #print( 'Parameters Received<br />' ) ;
  #foreach (param())
  #{
  #    print( "$_=", param($_), " " ) ;
  #}

  if ( param('restart') )
  {
    # Starting over again on a specific account.
    my $restartName = param('restart') ;

    if ($restartName eq 'NewAccount')
    {
      # For CGI parameters we use no-space version of NewAccount.
      # But the Popup and later stuff requires the version with a space.
      $restartName='New Account' ;
    }

    # Clear all CGI parameters for a fresh start.
    CGI::delete_all() ;

    $popupDefault=$restartName ;
    goto paramDone ;
  }


  my $pw1 = param('Password1' ) ;
  my $pw2 = param('Password2' ) ;

  if ( param('account')  and (param('account') eq 'New Account') )
  {
    #This is a new account request.

    # What do we check for?\
    #   o Account name not blank
    #   o Account name valid (Only Ascii Chars > " " and <= '~'
    #   o Account name not already used
    #   o Password1 entered.
    #   o Password2 matches password1.

    # Extract the account name, and trim leading/trailing spaces.
    my $newname=param('AccountName') ;
    $newname =~ s/^ *// ;
    $newname =~ s/ *$// ;

    if ($newname eq '')
    { $error='An Account Name must be specified' ; 
        goto paramDone ;
    }
    # Check if name contains characters from 0x00 to 0x20(=space)
    # or above tilde(=0x7e) 
    elsif ( $newname =~ m/.*[\x00-\x20\x7f-\xff].*/ )
    { $error='This is an invalid Account Name' ; 
        goto paramDone ;
    }
    elsif (grep /^$newname$/, keys %accounts )
    { $error='That Account Name is already in use.' ; 
        goto paramDone ;
    }
    elsif ( $pw1 eq '' )
    { $error='A Password must be specified.' ; 
        goto paramDone ;
    }
    elsif ( $pw1 ne $pw2 )
    { $error='Passwords do not match.' ;
      param('Password1','' ) ;
      param('Password2','' ) ;
        goto paramDone ;
    }

    # Update the database.
    $accounts{$newname}{'password'} = makepw( $pw1 ) ;

    # The checkbox parameters are only transmitted from the form if
    # they are checked.  If not transmitted then param() returns false
    # for the parameter.
    $accounts{$newname}{'sysadmin'} = ( param('SysAdmin') ? 1 : 0 ) ;
    $accounts{$newname}{'gpsadmin'} = ( param('GPSadmin') ? 1 : 0 ) ;
    $accounts{$newname}{'fileaccess'} = ( param('FileAccess') ? 1 : 0 );

    # Write the new data into the configuration files.
    updateGuiAccounts( \%accounts ) ;
  }
  elsif ( param('delete') )
  {
    # A request has come in to delete an account.
    # Ignore it if this is the sysadmin account.  We never want that
    # to go away.  Otherwise, erase that account's info and update the
    # configuration files.

    my $accname = param('delete') ;

    if ($accname ne 'sysadmin')
    {
      delete $accounts{$accname} ;

      # Write the new data into the configuration files.
      updateGuiAccounts( \%accounts ) ;
    }
  }
  else
  {
    # This is a request to modify an existing account.
    # Either the password is changing, or the permissions are changing,
    # or both.
    my $accname=param('account') ;
    $popupDefault=$accname ;

    if ( ($pw1 ne '') || ($pw2 ne '') )
    {
      if ( $pw1 ne $pw2 )
      { 
        $error='Passwords do not match.' ;
        param('Password1','' ) ; # Erase these fields in the form.
        param('Password2','' ) ;
        goto paramDone ;
      }

      # Encrypt the new password.
      $accounts{$accname}{'password'} = makepw( $pw1 ) ;
    }

    # Set the capabilities for this account.  
    # The checkbox parameters are only transmitted from the form if
    # they are checked.  If not transmitted then param() returns false
    # for the parameter.
    $accounts{$accname}{'sysadmin'} = ( param('SysAdmin') ? 1 : 0 ) ;
    $accounts{$accname}{'gpsadmin'} = ( param('GPSadmin') ? 1 : 0 ) ;
    $accounts{$accname}{'fileaccess'} = ( param('FileAccess') ? 1 : 0 );

    # Always keep full permissions for the main sysadmin account.
    $accounts{'sysadmin'}{'sysadmin'} = 1 ;
    $accounts{'sysadmin'}{'gpsadmin'} = 1 ;
    $accounts{'sysadmin'}{'fileaccess'} = 1 ;

    updateGuiAccounts( \%accounts ) ;
  }

  # Nothing went wrong with the parameters, so valid new account 
  # information was received and acted upon.  

  # Display a message indicating acceptance
  print h1('Account Changes Accepted.') ;

  # After a timeout, go back to main Accounts screen.
  print '
         Switching to Account Information display.
          
         <Script>
         setTimeout( \'document.location = "/perl-scripts/sysadmin/accounts.cgi"\',1000) ;
         </Script>' ;

  # Done with page.  Skip the rest of this script.  
  print end_html() ;
  goto EXIT ;

} # End of "if cgi parameters came in".

paramDone:

# When we get to here, either this is a first invocation of the script,
# or something was wrong with the user's selections.  Redraw the
# forms (perhaps with an error message alert) and wait for further
# input.


# Create the form with fields for account data entry.
print startform( {name=>'theForm'} ) ;

# Embed some javascript functions into the HTML page.
print <<EOF ;
<Script>

function restart()
/* The account to modify is selected by a popupmenu, which is populated
 * with 'New Account' and all of the existing accounts.  When this
/* selector is changed, we re-format the entire screen with that
 * particular account's information.  This is triggered by an onChange
 * condition on the popup, which calls this javascript function.
 * This function calls the cgi script with the special parameter
 * 'restart=accountname'.
 * In the special case of the 'New Account' item on the popup, we 
 * send the account name without the space character.  That avoids
 * some HTML nasties.
 */
{
  selectedIndex=document.theForm.account.selectedIndex ;
  selectedName=document.theForm.account.options[selectedIndex].text ;
  if (selectedIndex == 0)
    selectedName = 'NewAccount' ;
  parent.loadDataFrame(  
                       '/perl-scripts/sysadmin/editAccount.cgi?restart='
                       + selectedName
                      ) ;
}

function deleteQuery()
/* When the user presses the 'Delete Account' button, this function
 * is called.  The name of the account being displayed is extracted from
 * the form.  Then a confirmation dialog box is displayed asking if
 * that's really what the user wants.  After confirmation we post the
 * request by calling the URL with ?delete=acctname as the CGI
 * parameter.
 */
{
  selectedIndex=document.theForm.account.selectedIndex ;
  selectedName=document.theForm.account.options[selectedIndex].text ;
  if (confirm( 'Delete Account ' + selectedName ) )
  {
    parent.loadDataFrame( 
                        '/perl-scripts/sysadmin/editAccount.cgi?delete='
                        + selectedName
                        ) ;
  }
}
</Script>
EOF



my @values=('New Account') ;
push @values, (sort keys %accounts) ;

# The basic form we are after is a two-column table, split down the
# middle of the panel.  On the left are labels of the entry fields,
# right aligned in their cells.  On the right are the various widgets
# needed for specifying the account characteristics.
# The last row in the table is used for the action buttons, of which
# there can be four whose labels depend on the type of of form this
# is -- new account versus existing, etc.  Those are 
print '<Table align=center width=100% >' ;

print Tr( th( {align=>'right'}, 'Select Account' ),
          td( {align=>'left'},
              popup_menu( -name=>'account',
                  -values=>\@values,
                  -onChange=>'restart() ;',
                  -default=>$popupDefault
                ) ) ) ;


if ( $popupDefault eq 'New Account' )
{
print Tr( th( {-align=>'right'}, 'Account Name' ),
          td( {-align=>'left'},
              textfield( 'AccountName', '', 12, 12 ) )
        ) ;
}

print Tr( th( {-align=>'right'}, 'Password' ),
          td( password_field( 'Password1', '', 12, 12 ) )
        ) ;

print Tr( th( {-align=>'right', -width=>'50%'}, 'Re-Enter Password' ),
          td( {-align=>'left', -width=>'50%'},
              password_field( 'Password2', '', 12, 12 ) )
        ) ;

if ( ($popupDefault ne 'sysadmin') )
{
print Tr( th( {rowspan=>3, align=>'right'}, 'Access<br />Rights' ),
          td( checkbox( -name=>'SysAdmin',
                        -value=>'yes',
                        -label=>' System Controls',
                        -checked=>($accounts{$popupDefault}{'sysadmin'})
                      )
            )
        ) ;

print Tr( td( checkbox( -name=>'GPSadmin',
                        -value=>'yes',
                        -label=>' GPS Controls',
                        -checked=>($accounts{$popupDefault}{'gpsadmin'})
                      )
            )
        ) ;

print Tr( td( checkbox( -name=>'FileAccess',
                        -value=>'yes',
                        -label=>' File Access',
                        -checked=>($accounts{$popupDefault}{'fileaccess'})
                      )
            )
        ) ;
}        

# Create the action buttons along the bottom of the form.  The exact
# combination depends on the type of account being set up.
# For a 'New Account' we show 
#
#               Create      Reset       Cancel
#               Account     Form    
#
# For existing accounts, we show
#
#   Delete      Modify      Reset       Cancel
#   Account     Account     Form
#
# In the case of a sysadmin account, the 'Delete' button is suppressed.

if ( $popupDefault eq 'New Account' )
{
  print Tr( 
          # Left cell of row gets the Create button.
          td( {align=>'right'},
              submit( 'create', 'Create Account' ) 
            ),

          # Right cell of row gets the Reset and Cancel buttons
          td( button( -name=>'reset', -value=>'Reset Form', 
                      -onClick=> "parent.loadDataFrame( 
          '/perl-scripts/sysadmin/editAccount.cgi?restart=$popupDefault'
                                                      ) " 
                    ) 
              ,
              button( -name=>"cancel", -value=>"Cancel", 
                      -onClick=>
                  'parent.loadDataFrame( "/perl-scripts/sysadmin/accounts.cgi" )'
                    ) 
            )
          ) ;
}
else
{
  # Modifying an existing account.  
  # On the left are Delete and Modify buttons.
  # On the right are Reset and Cancel.
  #
  # For the Delete Button, we want to position it way to the left,
  # away from the other buttons, to avoid accidental clicks.  to
  # implement this the cell is filled with a sub-table for the two
  # buttons, allow the alignment to be specified.
  #
  print Tr( 
            td( table( {width=>"100%"},
                td( {align=>'left'}, 
                    ( $popupDefault ne "sysadmin" )
                    ? button( -name=>"delete", 
                                 -value=>"Delete Account", 
                                 -onClick=>'deleteQuery();'
                            ) 
                    : " " 
                  ) ,
                td( {align=>'right'}, submit( 'create', 
                    ( $popupDefault ne 'New Account' )
                    ? 'Accept Changes' 
                    : 'Create Account'
                    )
                  ) 
            ) ),

            td( button( -name=>'reset', 
                        -value=>'Reset Form', 
                        -onClick=> "parent.loadDataFrame( 
          '/perl-scripts/sysadmin/editAccount.cgi?restart=$popupDefault'
                                                        )"
                      ) ,
                button( -name=>'cancel', 
                        -value=>'Cancel', 
                        -onClick=>
                  'parent.loadDataFrame( "/perl-scripts/sysadmin/accounts.cgi" )'
                      ) 
              )
          ) ;
}


# End of table.  End of form.
print "\n</Table>\n</Form>\n" ;


# If an error message was found earlier during CGI processing,
# post an alert message to the user.
if ( length $error )
{
  print "<Script> 
  alert( \"$error\" ) ;
  </Script>
  " ;

  # Also display the message on the bottom of the page, in case
  # the javascript alert system doesn't work.
  print "$error" ;
}

print end_html() ;

EXIT:
########################################################################
#end of script editAccount.cgi
########################################################################
